@@ -1,9 +1,10 @@
 # orm/collections.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: allow-untyped-defs, allow-untyped-calls
 
 """Support for collections of mapped entities.
 
@@ -20,7 +21,7 @@ provided.  One is a bundle of generic decorators that map function arguments
 and return values to events::
 
   from sqlalchemy.orm.collections import collection
-  class MyClass(object):
+  class MyClass:
       # ...
 
       @collection.adds(1)
@@ -102,204 +103,83 @@ The owning object and :class:`.CollectionAttributeImpl` are also reachable
 through the adapter, allowing for some very sophisticated behavior.
 
 """
+from __future__ import annotations
 
 import operator
+import threading
+import typing
+from typing import Any
+from typing import Callable
+from typing import cast
+from typing import Collection
+from typing import Dict
+from typing import Iterable
+from typing import List
+from typing import NoReturn
+from typing import Optional
+from typing import Set
+from typing import Tuple
+from typing import Type
+from typing import TYPE_CHECKING
+from typing import TypeVar
+from typing import Union
 import weakref
 
-from sqlalchemy.util.compat import inspect_getfullargspec
-from . import base
+from .base import NO_KEY
 from .. import exc as sa_exc
 from .. import util
-from ..sql import coercions
-from ..sql import expression
-from ..sql import roles
+from ..sql.base import NO_ARG
+from ..util.compat import inspect_getfullargspec
+from ..util.typing import Protocol
+
+if typing.TYPE_CHECKING:
+    from .attributes import AttributeEventToken
+    from .attributes import CollectionAttributeImpl
+    from .mapped_collection import attribute_keyed_dict
+    from .mapped_collection import column_keyed_dict
+    from .mapped_collection import keyfunc_mapping
+    from .mapped_collection import KeyFuncDict  # noqa: F401
+    from .state import InstanceState
+
 
 __all__ = [
     "collection",
     "collection_adapter",
-    "mapped_collection",
-    "column_mapped_collection",
-    "attribute_mapped_collection",
+    "keyfunc_mapping",
+    "column_keyed_dict",
+    "attribute_keyed_dict",
+    "column_keyed_dict",
+    "attribute_keyed_dict",
+    "MappedCollection",
+    "KeyFuncDict",
 ]
 
-__instrumentation_mutex = util.threading.Lock()
+__instrumentation_mutex = threading.Lock()
 
 
-class _PlainColumnGetter(object):
-    """Plain column getter, stores collection of Column objects
-    directly.
+_CollectionFactoryType = Callable[[], "_AdaptedCollectionProtocol"]
 
-    Serializes to a :class:`._SerializableColumnGetterV2`
-    which has more expensive __call__() performance
-    and some rare caveats.
-
-    """
-
-    def __init__(self, cols):
-        self.cols = cols
-        self.composite = len(cols) > 1
-
-    def __reduce__(self):
-        return _SerializableColumnGetterV2._reduce_from_cols(self.cols)
-
-    def _cols(self, mapper):
-        return self.cols
-
-    def __call__(self, value):
-        state = base.instance_state(value)
-        m = base._state_mapper(state)
-
-        key = [
-            m._get_state_attr_by_column(state, state.dict, col)
-            for col in self._cols(m)
-        ]
-
-        if self.composite:
-            return tuple(key)
-        else:
-            return key[0]
+_T = TypeVar("_T", bound=Any)
+_KT = TypeVar("_KT", bound=Any)
+_VT = TypeVar("_VT", bound=Any)
+_COL = TypeVar("_COL", bound="Collection[Any]")
+_FN = TypeVar("_FN", bound="Callable[..., Any]")
 
 
-class _SerializableColumnGetter(object):
-    """Column-based getter used in version 0.7.6 only.
-
-    Remains here for pickle compatibility with 0.7.6.
-
-    """
-
-    def __init__(self, colkeys):
-        self.colkeys = colkeys
-        self.composite = len(colkeys) > 1
-
-    def __reduce__(self):
-        return _SerializableColumnGetter, (self.colkeys,)
-
-    def __call__(self, value):
-        state = base.instance_state(value)
-        m = base._state_mapper(state)
-        key = [
-            m._get_state_attr_by_column(
-                state, state.dict, m.mapped_table.columns[k]
-            )
-            for k in self.colkeys
-        ]
-        if self.composite:
-            return tuple(key)
-        else:
-            return key[0]
+class _CollectionConverterProtocol(Protocol):
+    def __call__(self, collection: _COL) -> _COL:
+        ...
 
 
-class _SerializableColumnGetterV2(_PlainColumnGetter):
-    """Updated serializable getter which deals with
-    multi-table mapped classes.
-
-    Two extremely unusual cases are not supported.
-    Mappings which have tables across multiple metadata
-    objects, or which are mapped to non-Table selectables
-    linked across inheriting mappers may fail to function
-    here.
-
-    """
-
-    def __init__(self, colkeys):
-        self.colkeys = colkeys
-        self.composite = len(colkeys) > 1
-
-    def __reduce__(self):
-        return self.__class__, (self.colkeys,)
-
-    @classmethod
-    def _reduce_from_cols(cls, cols):
-        def _table_key(c):
-            if not isinstance(c.table, expression.TableClause):
-                return None
-            else:
-                return c.table.key
-
-        colkeys = [(c.key, _table_key(c)) for c in cols]
-        return _SerializableColumnGetterV2, (colkeys,)
-
-    def _cols(self, mapper):
-        cols = []
-        metadata = getattr(mapper.local_table, "metadata", None)
-        for (ckey, tkey) in self.colkeys:
-            if tkey is None or metadata is None or tkey not in metadata:
-                cols.append(mapper.local_table.c[ckey])
-            else:
-                cols.append(metadata.tables[tkey].c[ckey])
-        return cols
+class _AdaptedCollectionProtocol(Protocol):
+    _sa_adapter: CollectionAdapter
+    _sa_appender: Callable[..., Any]
+    _sa_remover: Callable[..., Any]
+    _sa_iterator: Callable[..., Iterable[Any]]
+    _sa_converter: _CollectionConverterProtocol
 
 
-def column_mapped_collection(mapping_spec):
-    """A dictionary-based collection type with column-based keying.
-
-    Returns a :class:`.MappedCollection` factory with a keying function
-    generated from mapping_spec, which may be a Column or a sequence
-    of Columns.
-
-    The key value must be immutable for the lifetime of the object.  You
-    can not, for example, map on foreign key values if those key values will
-    change during the session, i.e. from None to a database-assigned integer
-    after a session flush.
-
-    """
-    cols = [
-        coercions.expect(roles.ColumnArgumentRole, q, argname="mapping_spec")
-        for q in util.to_list(mapping_spec)
-    ]
-    keyfunc = _PlainColumnGetter(cols)
-    return lambda: MappedCollection(keyfunc)
-
-
-class _SerializableAttrGetter(object):
-    def __init__(self, name):
-        self.name = name
-        self.getter = operator.attrgetter(name)
-
-    def __call__(self, target):
-        return self.getter(target)
-
-    def __reduce__(self):
-        return _SerializableAttrGetter, (self.name,)
-
-
-def attribute_mapped_collection(attr_name):
-    """A dictionary-based collection type with attribute-based keying.
-
-    Returns a :class:`.MappedCollection` factory with a keying based on the
-    'attr_name' attribute of entities in the collection, where ``attr_name``
-    is the string name of the attribute.
-
-    .. warning:: the key value must be assigned to its final value
-       **before** it is accessed by the attribute mapped collection.
-       Additionally, changes to the key attribute are **not tracked**
-       automatically, which means the key in the dictionary is not
-       automatically synchronized with the key value on the target object
-       itself.  See the section :ref:`key_collections_mutations`
-       for an example.
-
-    """
-    getter = _SerializableAttrGetter(attr_name)
-    return lambda: MappedCollection(getter)
-
-
-def mapped_collection(keyfunc):
-    """A dictionary-based collection type with arbitrary keying.
-
-    Returns a :class:`.MappedCollection` factory with a keying function
-    generated from keyfunc, a callable that takes an entity and returns a
-    key value.
-
-    The key value must be immutable for the lifetime of the object.  You
-    can not, for example, map on foreign key values if those key values will
-    change during the session, i.e. from None to a database-assigned integer
-    after a session flush.
-
-    """
-    return lambda: MappedCollection(keyfunc)
-
-
-class collection(object):
+class collection:
     """Decorators for entity collection classes.
 
     The decorators fall into two groups: annotations and interception recipes.
@@ -563,11 +443,16 @@ class collection(object):
         return decorator
 
 
-collection_adapter = operator.attrgetter("_sa_adapter")
-"""Fetch the :class:`.CollectionAdapter` for a collection."""
+if TYPE_CHECKING:
+
+    def collection_adapter(collection: Collection[Any]) -> CollectionAdapter:
+        """Fetch the :class:`.CollectionAdapter` for a collection."""
+
+else:
+    collection_adapter = operator.attrgetter("_sa_adapter")
 
 
-class CollectionAdapter(object):
+class CollectionAdapter:
     """Bridges between the ORM and arbitrary Python collections.
 
     Proxies base-level collection operations (append, remove, iterate)
@@ -590,26 +475,49 @@ class CollectionAdapter(object):
         "empty",
     )
 
-    def __init__(self, attr, owner_state, data):
+    attr: CollectionAttributeImpl
+    _key: str
+
+    # this is actually a weakref; see note in constructor
+    _data: Callable[..., _AdaptedCollectionProtocol]
+
+    owner_state: InstanceState[Any]
+    _converter: _CollectionConverterProtocol
+    invalidated: bool
+    empty: bool
+
+    def __init__(
+        self,
+        attr: CollectionAttributeImpl,
+        owner_state: InstanceState[Any],
+        data: _AdaptedCollectionProtocol,
+    ):
         self.attr = attr
         self._key = attr.key
-        self._data = weakref.ref(data)
+
+        # this weakref stays referenced throughout the lifespan of
+        # CollectionAdapter.  so while the weakref can return None, this
+        # is realistically only during garbage collection of this object, so
+        # we type this as a callable that returns _AdaptedCollectionProtocol
+        # in all cases.
+        self._data = weakref.ref(data)  # type: ignore
+
         self.owner_state = owner_state
         data._sa_adapter = self
         self._converter = data._sa_converter
         self.invalidated = False
         self.empty = False
 
-    def _warn_invalidated(self):
+    def _warn_invalidated(self) -> None:
         util.warn("This collection has been invalidated.")
 
     @property
-    def data(self):
+    def data(self) -> _AdaptedCollectionProtocol:
         "The entity collection being adapted."
         return self._data()
 
     @property
-    def _referenced_by_owner(self):
+    def _referenced_by_owner(self) -> bool:
         """return True if the owner state still refers to this collection.
 
         This will return False within a bulk replace operation,
@@ -621,7 +529,9 @@ class CollectionAdapter(object):
     def bulk_appender(self):
         return self._data()._sa_appender
 
-    def append_with_event(self, item, initiator=None):
+    def append_with_event(
+        self, item: Any, initiator: Optional[AttributeEventToken] = None
+    ) -> None:
         """Add an entity to the collection, firing mutation events."""
 
         self._data()._sa_appender(item, _sa_initiator=initiator)
@@ -633,7 +543,7 @@ class CollectionAdapter(object):
         self.empty = True
         self.owner_state._empty_collections[self._key] = user_data
 
-    def _reset_empty(self):
+    def _reset_empty(self) -> None:
         assert (
             self.empty
         ), "This collection adapter is not in the 'empty' state"
@@ -642,20 +552,20 @@ class CollectionAdapter(object):
             self._key
         ] = self.owner_state._empty_collections.pop(self._key)
 
-    def _refuse_empty(self):
+    def _refuse_empty(self) -> NoReturn:
         raise sa_exc.InvalidRequestError(
             "This is a special 'empty' collection which cannot accommodate "
             "internal mutation operations"
         )
 
-    def append_without_event(self, item):
+    def append_without_event(self, item: Any) -> None:
         """Add or restore an entity to the collection, firing no events."""
 
         if self.empty:
             self._refuse_empty()
         self._data()._sa_appender(item, _sa_initiator=False)
 
-    def append_multiple_without_event(self, items):
+    def append_multiple_without_event(self, items: Iterable[Any]) -> None:
         """Add or restore an entity to the collection, firing no events."""
         if self.empty:
             self._refuse_empty()
@@ -666,17 +576,21 @@ class CollectionAdapter(object):
     def bulk_remover(self):
         return self._data()._sa_remover
 
-    def remove_with_event(self, item, initiator=None):
+    def remove_with_event(
+        self, item: Any, initiator: Optional[AttributeEventToken] = None
+    ) -> None:
         """Remove an entity from the collection, firing mutation events."""
         self._data()._sa_remover(item, _sa_initiator=initiator)
 
-    def remove_without_event(self, item):
+    def remove_without_event(self, item: Any) -> None:
         """Remove an entity from the collection, firing no events."""
         if self.empty:
             self._refuse_empty()
         self._data()._sa_remover(item, _sa_initiator=False)
 
-    def clear_with_event(self, initiator=None):
+    def clear_with_event(
+        self, initiator: Optional[AttributeEventToken] = None
+    ) -> None:
         """Empty the collection, firing a mutation event for each entity."""
 
         if self.empty:
@@ -685,7 +599,7 @@ class CollectionAdapter(object):
         for item in list(self):
             remover(item, _sa_initiator=initiator)
 
-    def clear_without_event(self):
+    def clear_without_event(self) -> None:
         """Empty the collection, firing no events."""
 
         if self.empty:
@@ -706,9 +620,7 @@ class CollectionAdapter(object):
     def __bool__(self):
         return True
 
-    __nonzero__ = __bool__
-
-    def fire_append_wo_mutation_event(self, item, initiator=None):
+    def fire_append_wo_mutation_event(self, item, initiator=None, key=NO_KEY):
         """Notify that a entity is entering the collection but is already
         present.
 
@@ -729,12 +641,12 @@ class CollectionAdapter(object):
                 self._reset_empty()
 
             return self.attr.fire_append_wo_mutation_event(
-                self.owner_state, self.owner_state.dict, item, initiator
+                self.owner_state, self.owner_state.dict, item, initiator, key
             )
         else:
             return item
 
-    def fire_append_event(self, item, initiator=None):
+    def fire_append_event(self, item, initiator=None, key=NO_KEY):
         """Notify that a entity has entered the collection.
 
         Initiator is a token owned by the InstrumentedAttribute that
@@ -751,12 +663,12 @@ class CollectionAdapter(object):
                 self._reset_empty()
 
             return self.attr.fire_append_event(
-                self.owner_state, self.owner_state.dict, item, initiator
+                self.owner_state, self.owner_state.dict, item, initiator, key
             )
         else:
             return item
 
-    def fire_remove_event(self, item, initiator=None):
+    def fire_remove_event(self, item, initiator=None, key=NO_KEY):
         """Notify that a entity has been removed from the collection.
 
         Initiator is the InstrumentedAttribute that initiated the membership
@@ -772,10 +684,10 @@ class CollectionAdapter(object):
                 self._reset_empty()
 
             self.attr.fire_remove_event(
-                self.owner_state, self.owner_state.dict, item, initiator
+                self.owner_state, self.owner_state.dict, item, initiator, key
             )
 
-    def fire_pre_remove_event(self, initiator=None):
+    def fire_pre_remove_event(self, initiator=None, key=NO_KEY):
         """Notify that an entity is about to be removed from the collection.
 
         Only called if the entity cannot be removed after calling
@@ -785,7 +697,10 @@ class CollectionAdapter(object):
         if self.invalidated:
             self._warn_invalidated()
         self.attr.fire_pre_remove_event(
-            self.owner_state, self.owner_state.dict, initiator=initiator
+            self.owner_state,
+            self.owner_state.dict,
+            initiator=initiator,
+            key=key,
         )
 
     def __getstate__(self):
@@ -801,7 +716,10 @@ class CollectionAdapter(object):
     def __setstate__(self, d):
         self._key = d["key"]
         self.owner_state = d["owner_state"]
-        self._data = weakref.ref(d["data"])
+
+        # see note in constructor regarding this type: ignore
+        self._data = weakref.ref(d["data"])  # type: ignore
+
         self._converter = d["data"]._sa_converter
         d["data"]._sa_adapter = self
         self.invalidated = d["invalidated"]
@@ -849,7 +767,9 @@ def bulk_replace(values, existing_adapter, new_adapter, initiator=None):
             existing_adapter.fire_remove_event(member, initiator=initiator)
 
 
-def prepare_instrumentation(factory):
+def prepare_instrumentation(
+    factory: Union[Type[Collection[Any]], _CollectionFactoryType],
+) -> _CollectionFactoryType:
     """Prepare a callable for future use as a collection class factory.
 
     Given a collection class factory (either a type or no-arg callable),
@@ -860,18 +780,30 @@ def prepare_instrumentation(factory):
     into the run-time behavior of collection_class=InstrumentedList.
 
     """
+
+    impl_factory: _CollectionFactoryType
+
     # Convert a builtin to 'Instrumented*'
     if factory in __canned_instrumentation:
-        factory = __canned_instrumentation[factory]
+        impl_factory = __canned_instrumentation[factory]
+    else:
+        impl_factory = cast(_CollectionFactoryType, factory)
+
+    cls: Union[_CollectionFactoryType, Type[Collection[Any]]]
 
     # Create a specimen
-    cls = type(factory())
+    cls = type(impl_factory())
 
     # Did factory callable return a builtin?
     if cls in __canned_instrumentation:
-        # Wrap it so that it returns our 'Instrumented*'
-        factory = __converting_factory(cls, factory)
-        cls = factory()
+
+        # if so, just convert.
+        # in previous major releases, this codepath wasn't working and was
+        # not covered by tests.   prior to that it supplied a "wrapper"
+        # function that would return the class, though the rationale for this
+        # case is not known
+        impl_factory = __canned_instrumentation[cls]
+        cls = type(impl_factory())
 
     # Instrument the class if needed.
     if __instrumentation_mutex.acquire():
@@ -881,26 +813,7 @@ def prepare_instrumentation(factory):
         finally:
             __instrumentation_mutex.release()
 
-    return factory
-
-
-def __converting_factory(specimen_cls, original_factory):
-    """Return a wrapper that converts a "canned" collection like
-    set, dict, list into the Instrumented* version.
-
-    """
-
-    instrumented_cls = __canned_instrumentation[specimen_cls]
-
-    def wrapper():
-        collection = original_factory()
-        return instrumented_cls(collection)
-
-    # often flawed but better than nothing
-    wrapper.__name__ = "%sWrapper" % original_factory.__name__
-    wrapper.__doc__ = original_factory.__doc__
-
-    return wrapper
+    return impl_factory
 
 
 def _instrument_class(cls):
@@ -930,8 +843,8 @@ def _locate_roles_and_methods(cls):
 
     """
 
-    roles = {}
-    methods = {}
+    roles: Dict[str, str] = {}
+    methods: Dict[str, Tuple[Optional[str], Optional[int], Optional[str]]] = {}
 
     for supercls in cls.__mro__:
         for name, method in vars(supercls).items():
@@ -951,7 +864,9 @@ def _locate_roles_and_methods(cls):
 
             # transfer instrumentation requests from decorated function
             # to the combined queue
-            before, after = None, None
+            before: Optional[Tuple[str, int]] = None
+            after: Optional[str] = None
+
             if hasattr(method, "_sa_instrument_before"):
                 op, argument = method._sa_instrument_before
                 assert op in ("fire_append_event", "fire_remove_event")
@@ -976,6 +891,7 @@ def _setup_canned_roles(cls, roles, methods):
     """
     collection_type = util.duck_type_collection(cls)
     if collection_type in __interfaces:
+        assert collection_type is not None
         canned_roles, decorators = __interfaces[collection_type]
         for role, name in canned_roles.items():
             roles.setdefault(role, name)
@@ -1101,9 +1017,9 @@ def _instrument_membership_mutator(method, before, argument, after):
                 getattr(executor, after)(res, initiator)
             return res
 
-    wrapper._sa_instrumented = True
+    wrapper._sa_instrumented = True  # type: ignore[attr-defined]
     if hasattr(method, "_sa_instrument_role"):
-        wrapper._sa_instrument_role = method._sa_instrument_role
+        wrapper._sa_instrument_role = method._sa_instrument_role  # type: ignore[attr-defined]  # noqa: E501
     wrapper.__name__ = method.__name__
     wrapper.__doc__ = method.__doc__
     return wrapper
@@ -1118,10 +1034,12 @@ def __set_wo_mutation(collection, item, _sa_initiator=None):
     if _sa_initiator is not False:
         executor = collection._sa_adapter
         if executor:
-            executor.fire_append_wo_mutation_event(item, _sa_initiator)
+            executor.fire_append_wo_mutation_event(
+                item, _sa_initiator, key=None
+            )
 
 
-def __set(collection, item, _sa_initiator=None):
+def __set(collection, item, _sa_initiator, key):
     """Run set events.
 
     This event always occurs before the collection is actually mutated.
@@ -1131,11 +1049,11 @@ def __set(collection, item, _sa_initiator=None):
     if _sa_initiator is not False:
         executor = collection._sa_adapter
         if executor:
-            item = executor.fire_append_event(item, _sa_initiator)
+            item = executor.fire_append_event(item, _sa_initiator, key=key)
     return item
 
 
-def __del(collection, item, _sa_initiator=None):
+def __del(collection, item, _sa_initiator, key):
     """Run del events.
 
     This event occurs before the collection is actually mutated, *except*
@@ -1147,7 +1065,7 @@ def __del(collection, item, _sa_initiator=None):
     if _sa_initiator is not False:
         executor = collection._sa_adapter
         if executor:
-            executor.fire_remove_event(item, _sa_initiator)
+            executor.fire_remove_event(item, _sa_initiator, key=key)
 
 
 def __before_pop(collection, _sa_initiator=None):
@@ -1157,7 +1075,7 @@ def __before_pop(collection, _sa_initiator=None):
         executor.fire_pre_remove_event(_sa_initiator)
 
 
-def _list_decorators():
+def _list_decorators() -> Dict[str, Callable[[_FN], _FN]]:
     """Tailored instrumentation wrappers for any list-like class."""
 
     def _tidy(fn):
@@ -1166,7 +1084,7 @@ def _list_decorators():
 
     def append(fn):
         def append(self, item, _sa_initiator=None):
-            item = __set(self, item, _sa_initiator)
+            item = __set(self, item, _sa_initiator, NO_KEY)
             fn(self, item)
 
         _tidy(append)
@@ -1174,7 +1092,7 @@ def _list_decorators():
 
     def remove(fn):
         def remove(self, value, _sa_initiator=None):
-            __del(self, value, _sa_initiator)
+            __del(self, value, _sa_initiator, NO_KEY)
             # testlib.pragma exempt:__eq__
             fn(self, value)
 
@@ -1183,7 +1101,7 @@ def _list_decorators():
 
     def insert(fn):
         def insert(self, index, value):
-            value = __set(self, value)
+            value = __set(self, value, None, index)
             fn(self, index, value)
 
         _tidy(insert)
@@ -1194,8 +1112,8 @@ def _list_decorators():
             if not isinstance(index, slice):
                 existing = self[index]
                 if existing is not None:
-                    __del(self, existing)
-                value = __set(self, value)
+                    __del(self, existing, None, index)
+                value = __set(self, value, None, index)
                 fn(self, index, value)
             else:
                 # slice assignment requires __delitem__, insert, __len__
@@ -1237,43 +1155,22 @@ def _list_decorators():
         def __delitem__(self, index):
             if not isinstance(index, slice):
                 item = self[index]
-                __del(self, item)
+                __del(self, item, None, index)
                 fn(self, index)
             else:
                 # slice deletion requires __getslice__ and a slice-groking
                 # __getitem__ for stepped deletion
                 # note: not breaking this into atomic dels
                 for item in self[index]:
-                    __del(self, item)
+                    __del(self, item, None, index)
                 fn(self, index)
 
         _tidy(__delitem__)
         return __delitem__
 
-    if util.py2k:
-
-        def __setslice__(fn):
-            def __setslice__(self, start, end, values):
-                for value in self[start:end]:
-                    __del(self, value)
-                values = [__set(self, value) for value in values]
-                fn(self, start, end, values)
-
-            _tidy(__setslice__)
-            return __setslice__
-
-        def __delslice__(fn):
-            def __delslice__(self, start, end):
-                for value in self[start:end]:
-                    __del(self, value)
-                fn(self, start, end)
-
-            _tidy(__delslice__)
-            return __delslice__
-
     def extend(fn):
         def extend(self, iterable):
-            for value in iterable:
+            for value in list(iterable):
                 self.append(value)
 
         _tidy(extend)
@@ -1283,7 +1180,7 @@ def _list_decorators():
         def __iadd__(self, iterable):
             # list.__iadd__ takes any iterable and seems to let TypeError
             # raise as-is instead of returning NotImplemented
-            for value in iterable:
+            for value in list(iterable):
                 self.append(value)
             return self
 
@@ -1294,22 +1191,20 @@ def _list_decorators():
         def pop(self, index=-1):
             __before_pop(self)
             item = fn(self, index)
-            __del(self, item)
+            __del(self, item, None, index)
             return item
 
         _tidy(pop)
         return pop
 
-    if not util.py2k:
+    def clear(fn):
+        def clear(self, index=-1):
+            for item in self:
+                __del(self, item, None, index)
+            fn(self)
 
-        def clear(fn):
-            def clear(self, index=-1):
-                for item in self:
-                    __del(self, item)
-                fn(self)
-
-            _tidy(clear)
-            return clear
+        _tidy(clear)
+        return clear
 
     # __imul__ : not wrapping this.  all members of the collection are already
     # present, so no need to fire appends... wrapping it with an explicit
@@ -1321,20 +1216,18 @@ def _list_decorators():
     return l
 
 
-def _dict_decorators():
+def _dict_decorators() -> Dict[str, Callable[[_FN], _FN]]:
     """Tailored instrumentation wrappers for any dict-like mapping class."""
 
     def _tidy(fn):
         fn._sa_instrumented = True
         fn.__doc__ = getattr(dict, fn.__name__).__doc__
 
-    Unspecified = util.symbol("Unspecified")
-
     def __setitem__(fn):
         def __setitem__(self, key, value, _sa_initiator=None):
             if key in self:
-                __del(self, self[key], _sa_initiator)
-            value = __set(self, value, _sa_initiator)
+                __del(self, self[key], _sa_initiator, key)
+            value = __set(self, value, _sa_initiator, key)
             fn(self, key, value)
 
         _tidy(__setitem__)
@@ -1343,7 +1236,7 @@ def _dict_decorators():
     def __delitem__(fn):
         def __delitem__(self, key, _sa_initiator=None):
             if key in self:
-                __del(self, self[key], _sa_initiator)
+                __del(self, self[key], _sa_initiator, key)
             fn(self, key)
 
         _tidy(__delitem__)
@@ -1352,22 +1245,22 @@ def _dict_decorators():
     def clear(fn):
         def clear(self):
             for key in self:
-                __del(self, self[key])
+                __del(self, self[key], None, key)
             fn(self)
 
         _tidy(clear)
         return clear
 
     def pop(fn):
-        def pop(self, key, default=Unspecified):
+        def pop(self, key, default=NO_ARG):
             __before_pop(self)
             _to_del = key in self
-            if default is Unspecified:
+            if default is NO_ARG:
                 item = fn(self, key)
             else:
                 item = fn(self, key, default)
             if _to_del:
-                __del(self, item)
+                __del(self, item, None, key)
             return item
 
         _tidy(pop)
@@ -1377,7 +1270,7 @@ def _dict_decorators():
         def popitem(self):
             __before_pop(self)
             item = fn(self)
-            __del(self, item[1])
+            __del(self, item[1], None, 1)
             return item
 
         _tidy(popitem)
@@ -1399,8 +1292,8 @@ def _dict_decorators():
         return setdefault
 
     def update(fn):
-        def update(self, __other=Unspecified, **kw):
-            if __other is not Unspecified:
+        def update(self, __other=NO_ARG, **kw):
+            if __other is not NO_ARG:
                 if hasattr(__other, "keys"):
                     for key in list(__other):
                         if key not in self or self[key] is not __other[key]:
@@ -1424,20 +1317,19 @@ def _dict_decorators():
 
     l = locals().copy()
     l.pop("_tidy")
-    l.pop("Unspecified")
     return l
 
 
 _set_binop_bases = (set, frozenset)
 
 
-def _set_binops_check_strict(self, obj):
+def _set_binops_check_strict(self: Any, obj: Any) -> bool:
     """Allow only set, frozenset and self.__class__-derived
     objects in binops."""
     return isinstance(obj, _set_binop_bases + (self.__class__,))
 
 
-def _set_binops_check_loose(self, obj):
+def _set_binops_check_loose(self: Any, obj: Any) -> bool:
     """Allow anything set-like to participate in set binops."""
     return (
         isinstance(obj, _set_binop_bases + (self.__class__,))
@@ -1445,19 +1337,17 @@ def _set_binops_check_loose(self, obj):
     )
 
 
-def _set_decorators():
+def _set_decorators() -> Dict[str, Callable[[_FN], _FN]]:
     """Tailored instrumentation wrappers for any set-like class."""
 
     def _tidy(fn):
         fn._sa_instrumented = True
         fn.__doc__ = getattr(set, fn.__name__).__doc__
 
-    Unspecified = util.symbol("Unspecified")
-
     def add(fn):
         def add(self, value, _sa_initiator=None):
             if value not in self:
-                value = __set(self, value, _sa_initiator)
+                value = __set(self, value, _sa_initiator, NO_KEY)
             else:
                 __set_wo_mutation(self, value, _sa_initiator)
             # testlib.pragma exempt:__hash__
@@ -1470,7 +1360,7 @@ def _set_decorators():
         def discard(self, value, _sa_initiator=None):
             # testlib.pragma exempt:__hash__
             if value in self:
-                __del(self, value, _sa_initiator)
+                __del(self, value, _sa_initiator, NO_KEY)
                 # testlib.pragma exempt:__hash__
             fn(self, value)
 
@@ -1481,7 +1371,7 @@ def _set_decorators():
         def remove(self, value, _sa_initiator=None):
             # testlib.pragma exempt:__hash__
             if value in self:
-                __del(self, value, _sa_initiator)
+                __del(self, value, _sa_initiator, NO_KEY)
             # testlib.pragma exempt:__hash__
             fn(self, value)
 
@@ -1494,7 +1384,7 @@ def _set_decorators():
             item = fn(self)
             # for set in particular, we have no way to access the item
             # that will be popped before pop is called.
-            __del(self, item)
+            __del(self, item, None, NO_KEY)
             return item
 
         _tidy(pop)
@@ -1606,101 +1496,82 @@ def _set_decorators():
 
     l = locals().copy()
     l.pop("_tidy")
-    l.pop("Unspecified")
     return l
 
 
-class InstrumentedList(list):
+class InstrumentedList(List[_T]):
     """An instrumented version of the built-in list."""
 
 
-class InstrumentedSet(set):
+class InstrumentedSet(Set[_T]):
     """An instrumented version of the built-in set."""
 
 
-class InstrumentedDict(dict):
+class InstrumentedDict(Dict[_KT, _VT]):
     """An instrumented version of the built-in dict."""
 
 
-__canned_instrumentation = {
-    list: InstrumentedList,
-    set: InstrumentedSet,
-    dict: InstrumentedDict,
-}
+__canned_instrumentation: util.immutabledict[
+    Any, _CollectionFactoryType
+] = util.immutabledict(
+    {
+        list: InstrumentedList,
+        set: InstrumentedSet,
+        dict: InstrumentedDict,
+    }
+)
 
-__interfaces = {
-    list: (
-        {"appender": "append", "remover": "remove", "iterator": "__iter__"},
-        _list_decorators(),
-    ),
-    set: (
-        {"appender": "add", "remover": "remove", "iterator": "__iter__"},
-        _set_decorators(),
-    ),
-    # decorators are required for dicts and object collections.
-    dict: ({"iterator": "values"}, _dict_decorators())
-    if util.py3k
-    else ({"iterator": "itervalues"}, _dict_decorators()),
-}
+__interfaces: util.immutabledict[
+    Any,
+    Tuple[
+        Dict[str, str],
+        Dict[str, Callable[..., Any]],
+    ],
+] = util.immutabledict(
+    {
+        list: (
+            {
+                "appender": "append",
+                "remover": "remove",
+                "iterator": "__iter__",
+            },
+            _list_decorators(),
+        ),
+        set: (
+            {"appender": "add", "remover": "remove", "iterator": "__iter__"},
+            _set_decorators(),
+        ),
+        # decorators are required for dicts and object collections.
+        dict: ({"iterator": "values"}, _dict_decorators()),
+    }
+)
 
 
-class MappedCollection(dict):
-    """A basic dictionary-based collection class.
+def __go(lcls):
 
-    Extends dict with the minimal bag semantics that collection
-    classes require. ``set`` and ``remove`` are implemented in terms
-    of a keying function: any callable that takes an object and
-    returns an object for use as a dictionary key.
+    global keyfunc_mapping, mapped_collection
+    global column_keyed_dict, column_mapped_collection
+    global MappedCollection, KeyFuncDict
+    global attribute_keyed_dict, attribute_mapped_collection
 
-    """
+    from .mapped_collection import keyfunc_mapping
+    from .mapped_collection import column_keyed_dict
+    from .mapped_collection import attribute_keyed_dict
+    from .mapped_collection import KeyFuncDict
 
-    def __init__(self, keyfunc):
-        """Create a new collection with keying provided by keyfunc.
+    from .mapped_collection import mapped_collection
+    from .mapped_collection import column_mapped_collection
+    from .mapped_collection import attribute_mapped_collection
+    from .mapped_collection import MappedCollection
 
-        keyfunc may be any callable that takes an object and returns an object
-        for use as a dictionary key.
-
-        The keyfunc will be called every time the ORM needs to add a member by
-        value-only (such as when loading instances from the database) or
-        remove a member.  The usual cautions about dictionary keying apply-
-        ``keyfunc(object)`` should return the same output for the life of the
-        collection.  Keying based on mutable properties can result in
-        unreachable instances "lost" in the collection.
-
-        """
-        self.keyfunc = keyfunc
-
-    @collection.appender
-    @collection.internally_instrumented
-    def set(self, value, _sa_initiator=None):
-        """Add an item by value, consulting the keyfunc for the key."""
-
-        key = self.keyfunc(value)
-        self.__setitem__(key, value, _sa_initiator)
-
-    @collection.remover
-    @collection.internally_instrumented
-    def remove(self, value, _sa_initiator=None):
-        """Remove an item by value, consulting the keyfunc for the key."""
-
-        key = self.keyfunc(value)
-        # Let self[key] raise if key is not in this collection
-        # testlib.pragma exempt:__ne__
-        if self[key] != value:
-            raise sa_exc.InvalidRequestError(
-                "Can not remove '%s': collection holds '%s' for key '%s'. "
-                "Possible cause: is the MappedCollection key function "
-                "based on mutable properties or properties that only obtain "
-                "values after flush?" % (value, self[key], key)
-            )
-        self.__delitem__(key, _sa_initiator)
+    # ensure instrumentation is associated with
+    # these built-in classes; if a user-defined class
+    # subclasses these and uses @internally_instrumented,
+    # the superclass is otherwise not instrumented.
+    # see [ticket:2406].
+    _instrument_class(InstrumentedList)
+    _instrument_class(InstrumentedSet)
+    _instrument_class(KeyFuncDict)
 
 
-# ensure instrumentation is associated with
-# these built-in classes; if a user-defined class
-# subclasses these and uses @internally_instrumented,
-# the superclass is otherwise not instrumented.
-# see [ticket:2406].
-_instrument_class(MappedCollection)
-_instrument_class(InstrumentedList)
-_instrument_class(InstrumentedSet)
+__go(locals())
