@@ -1,11 +1,9 @@
 from sqlalchemy import exc
 from sqlalchemy import testing
 from sqlalchemy.engine import result
-from sqlalchemy.engine.row import Row
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
-from sqlalchemy.testing import expect_deprecated
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_false
 from sqlalchemy.testing import is_true
@@ -196,6 +194,37 @@ class ResultTupleTest(fixtures.TestBase):
             eq_(kt._fields, ("a", "b"))
             eq_(kt._asdict(), {"a": 1, "b": 3})
 
+    @testing.requires.cextensions
+    def test_serialize_cy_py_cy(self):
+        from sqlalchemy.engine._py_row import BaseRow as _PyRow
+        from sqlalchemy.cyextension.resultproxy import BaseRow as _CyRow
+
+        global Row
+
+        p = result.SimpleResultMetaData(["a", None, "b"])
+
+        for loads, dumps in picklers():
+
+            class Row(_CyRow):
+                pass
+
+            row = Row(p, p._processors, p._keymap, 0, (1, 2, 3))
+
+            state = dumps(row)
+
+            class Row(_PyRow):
+                pass
+
+            row2 = loads(state)
+            is_true(isinstance(row2, _PyRow))
+            state2 = dumps(row2)
+
+            class Row(_CyRow):
+                pass
+
+            row3 = loads(state2)
+            is_true(isinstance(row3, _CyRow))
+
 
 class ResultTest(fixtures.TestBase):
     def _fixture(
@@ -224,6 +253,21 @@ class ResultTest(fixtures.TestBase):
 
         return res
 
+    def test_close_attributes(self):
+        """test #8710"""
+        r1 = self._fixture()
+
+        is_false(r1.closed)
+        is_false(r1._soft_closed)
+
+        r1._soft_close()
+        is_false(r1.closed)
+        is_true(r1._soft_closed)
+
+        r1.close()
+        is_true(r1.closed)
+        is_true(r1._soft_closed)
+
     def test_class_presented(self):
         """To support different kinds of objects returned vs. rows,
         there are two wrapper classes for Result.
@@ -247,6 +291,13 @@ class ResultTest(fixtures.TestBase):
         eq_(m1.fetchone(), {"a": 1, "b": 1, "c": 1})
         eq_(r1.fetchone(), (2, 1, 2))
 
+    def test_tuples_plus_base(self):
+        r1 = self._fixture()
+
+        t1 = r1.tuples()
+        eq_(t1.fetchone(), (1, 1, 1))
+        eq_(r1.fetchone(), (2, 1, 2))
+
     def test_scalar_plus_base(self):
         r1 = self._fixture()
 
@@ -494,7 +545,7 @@ class ResultTest(fixtures.TestBase):
 
     def test_one_unique(self):
         # assert that one() counts rows after uniqueness has been applied.
-        # this would raise if we didnt have unique
+        # this would raise if we didn't have unique
         result = self._fixture(data=[(1, 1, 1), (1, 1, 1)])
 
         row = result.unique().one()
@@ -511,7 +562,7 @@ class ResultTest(fixtures.TestBase):
 
     def test_one_unique_mapping(self):
         # assert that one() counts rows after uniqueness has been applied.
-        # this would raise if we didnt have unique
+        # this would raise if we didn't have unique
         result = self._fixture(data=[(1, 1, 1), (1, 1, 1)])
 
         row = result.mappings().unique().one()
@@ -698,27 +749,6 @@ class ResultTest(fixtures.TestBase):
         # still slices
         eq_(m1.fetchone(), {"b": 1, "c": 2})
 
-    def test_alt_row_fetch(self):
-        class AppleRow(Row):
-            def apple(self):
-                return "apple"
-
-        result = self._fixture(alt_row=AppleRow)
-
-        row = result.all()[0]
-        eq_(row.apple(), "apple")
-
-    def test_alt_row_transform(self):
-        class AppleRow(Row):
-            def apple(self):
-                return "apple"
-
-        result = self._fixture(alt_row=AppleRow)
-
-        row = result.columns("c", "a").all()[2]
-        eq_(row.apple(), "apple")
-        eq_(row, (2, 1))
-
     def test_scalar_none_iterate(self):
         result = self._fixture(
             data=[
@@ -1055,11 +1085,7 @@ class OnlyScalarsTest(fixtures.TestBase):
             metadata, no_tuple_fixture, source_supports_scalars=True
         )
 
-        with expect_deprecated(
-            r"The Result.columns\(\) method has a bug in SQLAlchemy 1.4 "
-            r"that is causing it to yield scalar values"
-        ):
-            r = r.columns(0).mappings()
+        r = r.columns(0).mappings()
         eq_(
             list(r),
             [{"a": 1}, {"a": 2}, {"a": 1}, {"a": 1}, {"a": 4}],
@@ -1074,15 +1100,10 @@ class OnlyScalarsTest(fixtures.TestBase):
             metadata, no_tuple_fixture, source_supports_scalars=True
         )
 
-        with expect_deprecated(
-            r"The Result.columns\(\) method has a bug in SQLAlchemy 1.4 "
-            r"that is causing it to yield scalar values"
-        ):
-            r = r.columns(0)
+        r = r.columns(0)
         eq_(
             list(r),
-            [1, 2, 1, 1, 4],
-            # [(1,), (2,), (1,), (1,), (4,)],  # correct result
+            [(1,), (2,), (1,), (1,), (4,)],
         )
 
     def test_scalar_mode_scalars0(self, no_tuple_fixture):
