@@ -1,5 +1,3 @@
-# coding: utf-8
-
 from sqlalchemy import Column
 from sqlalchemy import exc
 from sqlalchemy import Integer
@@ -206,6 +204,44 @@ class OnConflictTest(fixtures.TablesTest):
             [(1, "name1")],
         )
 
+    @testing.combinations(True, False, argnames="use_returning")
+    def test_on_conflict_do_update_set_executemany(
+        self, connection, use_returning
+    ):
+        """test #6581"""
+
+        users = self.tables.users
+
+        connection.execute(
+            users.insert(),
+            [dict(id=1, name="name1"), dict(id=2, name="name2")],
+        )
+
+        i = insert(users)
+        i = i.on_conflict_do_update(
+            index_elements=[users.c.id],
+            set_={"id": i.excluded.id, "name": i.excluded.name + ".5"},
+        )
+        if use_returning:
+            i = i.returning(users.c.id, users.c.name)
+
+        result = connection.execute(
+            i,
+            [
+                dict(id=1, name="name1"),
+                dict(id=2, name="name2"),
+                dict(id=3, name="name3"),
+            ],
+        )
+
+        if use_returning:
+            eq_(result.all(), [(1, "name1.5"), (2, "name2.5"), (3, "name3")])
+
+        eq_(
+            connection.execute(users.select().order_by(users.c.id)).fetchall(),
+            [(1, "name1.5"), (2, "name2.5"), (3, "name3")],
+        )
+
     def test_on_conflict_do_update_schema(self, connection):
         users = self.tables.get("%s.users_schema" % config.test_schema)
 
@@ -252,7 +288,7 @@ class OnConflictTest(fixtures.TablesTest):
     def test_on_conflict_do_update_clauseelem_as_key_set(self, connection):
         users = self.tables.users
 
-        class MyElem(object):
+        class MyElem:
             def __init__(self, expr):
                 self.expr = expr
 
@@ -646,6 +682,45 @@ class OnConflictTest(fixtures.TablesTest):
             [(1, "name1", "mail2@gmail.com", "unique_name")],
         )
 
+    def test_on_conflict_do_update_constraint_can_be_index(self, connection):
+        """test #9023"""
+
+        users = self.tables.users_xtra
+
+        connection.execute(
+            insert(users),
+            dict(
+                id=1,
+                name="name1",
+                login_email="mail1@gmail.com",
+                lets_index_this="unique_name",
+            ),
+        )
+
+        i = insert(users)
+        i = i.on_conflict_do_update(
+            constraint=self.unique_partial_index,
+            set_=dict(
+                name=i.excluded.name, login_email=i.excluded.login_email
+            ),
+        )
+
+        connection.execute(
+            i,
+            [
+                dict(
+                    name="name1",
+                    login_email="mail2@gmail.com",
+                    lets_index_this="unique_name",
+                )
+            ],
+        )
+
+        eq_(
+            connection.execute(users.select()).fetchall(),
+            [(1, "name1", "mail2@gmail.com", "unique_name")],
+        )
+
     def test_on_conflict_do_update_no_row_actually_affected(self, connection):
         users = self.tables.users_xtra
 
