@@ -1,4 +1,6 @@
 import time
+from unittest.mock import call
+from unittest.mock import Mock
 
 import sqlalchemy as tsa
 from sqlalchemy import create_engine
@@ -11,7 +13,7 @@ from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy import util
-from sqlalchemy.engine import url
+from sqlalchemy.engine import default
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import assert_raises_message_context_ok
@@ -19,7 +21,7 @@ from sqlalchemy.testing import assert_warns_message
 from sqlalchemy.testing import engines
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_raises
-from sqlalchemy.testing import expect_warnings
+from sqlalchemy.testing import expect_raises_message
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import is_false
@@ -27,9 +29,6 @@ from sqlalchemy.testing import is_true
 from sqlalchemy.testing import mock
 from sqlalchemy.testing import ne_
 from sqlalchemy.testing.engines import testing_engine
-from sqlalchemy.testing.mock import call
-from sqlalchemy.testing.mock import Mock
-from sqlalchemy.testing.mock import patch
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.testing.util import gc_collect
@@ -166,21 +165,20 @@ class PrePingMockTest(fixtures.TestBase):
     def setup_test(self):
         self.dbapi = MockDBAPI()
 
-    def _pool_fixture(self, pre_ping, pool_kw=None):
-        dialect = url.make_url(
-            "postgresql://foo:bar@localhost/test"
-        ).get_dialect()()
+    def _pool_fixture(self, pre_ping, setup_disconnect=True, pool_kw=None):
+        dialect = default.DefaultDialect()
         dialect.dbapi = self.dbapi
         _pool = pool.QueuePool(
             creator=lambda: self.dbapi.connect("foo.db"),
             pre_ping=pre_ping,
             dialect=dialect,
-            **(pool_kw if pool_kw else {})
+            **(pool_kw if pool_kw else {}),
         )
 
-        dialect.is_disconnect = lambda e, conn, cursor: isinstance(
-            e, MockDisconnect
-        )
+        if setup_disconnect:
+            dialect.is_disconnect = lambda e, conn, cursor: isinstance(
+                e, MockDisconnect
+            )
         return _pool
 
     def teardown_test(self):
@@ -263,6 +261,29 @@ class PrePingMockTest(fixtures.TestBase):
         stale_cursor = stale_connection.cursor()
         assert_raises(MockDisconnect, stale_cursor.execute, "hi")
 
+    def test_handle_error_sets_disconnect(self):
+        pool = self._pool_fixture(pre_ping=True, setup_disconnect=False)
+
+        @event.listens_for(pool._dialect, "handle_error")
+        def setup_disconnect(ctx):
+            assert isinstance(ctx.sqlalchemy_exception, exc.DBAPIError)
+            assert isinstance(ctx.original_exception, MockDisconnect)
+            ctx.is_disconnect = True
+
+        conn = pool.connect()
+        stale_connection = conn.dbapi_connection
+        conn.close()
+
+        self.dbapi.shutdown("execute")
+        self.dbapi.restart()
+
+        conn = pool.connect()
+        cursor = conn.cursor()
+        cursor.execute("hi")
+
+        stale_cursor = stale_connection.cursor()
+        assert_raises(MockDisconnect, stale_cursor.execute, "hi")
+
     def test_raise_db_is_stopped(self):
         pool = self._pool_fixture(pre_ping=True)
 
@@ -362,12 +383,12 @@ class MockReconnectTest(fixtures.TestBase):
         self.dbapi = MockDBAPI()
 
         self.db = testing_engine(
-            "postgresql://foo:bar@localhost/test",
+            "postgresql+psycopg2://foo:bar@localhost/test",
             options=dict(module=self.dbapi, _initialize=False),
         )
 
         self.mock_connect = call(
-            host="localhost", password="bar", user="foo", database="test"
+            host="localhost", password="bar", user="foo", dbname="test"
         )
         # monkeypatch disconnect checker
         self.db.dialect.is_disconnect = lambda e, conn, cursor: isinstance(
@@ -565,7 +586,8 @@ class MockReconnectTest(fixtures.TestBase):
         # error stays consistent
         assert_raises_message(
             tsa.exc.PendingRollbackError,
-            "This connection is on an inactive transaction.  Please rollback",
+            r"Can't reconnect until invalid transaction is rolled back.  "
+            r"Please rollback\(\) fully before proceeding",
             conn.execute,
             select(1),
         )
@@ -573,7 +595,8 @@ class MockReconnectTest(fixtures.TestBase):
 
         assert_raises_message(
             tsa.exc.PendingRollbackError,
-            "This connection is on an inactive transaction.  Please rollback",
+            r"Can't reconnect until invalid transaction is rolled back.  "
+            r"Please rollback\(\) fully before proceeding",
             trans.commit,
         )
 
@@ -581,7 +604,8 @@ class MockReconnectTest(fixtures.TestBase):
 
         assert_raises_message(
             tsa.exc.PendingRollbackError,
-            "This connection is on an inactive transaction.  Please rollback",
+            r"Can't reconnect until invalid transaction is rolled back.  "
+            r"Please rollback\(\) fully before proceeding",
             conn.execute,
             select(1),
         )
@@ -609,13 +633,22 @@ class MockReconnectTest(fixtures.TestBase):
 
         self.dbapi.shutdown()
 
-        assert_raises(tsa.exc.DBAPIError, conn.execute, select(1))
+        with expect_raises(tsa.exc.DBAPIError):
+            conn.execute(select(1))
 
         assert not conn.closed
         assert conn.invalidated
 
         eq_([c.close.mock_calls for c in self.dbapi.connections], [[call()]])
 
+        # trans was autobegin.  they have to call rollback
+        with expect_raises(tsa.exc.PendingRollbackError):
+            conn.execute(select(1))
+
+        # ok
+        conn.rollback()
+
+        # now we are good
         # test reconnects
         conn.execute(select(1))
         assert not conn.invalidated
@@ -642,8 +675,8 @@ class MockReconnectTest(fixtures.TestBase):
             select(1),
         )
 
-    def test_noreconnect_execute_plus_closewresult(self):
-        conn = self.db.connect(close_with_result=True)
+    def test_noreconnect_execute(self):
+        conn = self.db.connect()
 
         self.dbapi.shutdown("execute_no_disconnect")
 
@@ -655,31 +688,34 @@ class MockReconnectTest(fixtures.TestBase):
             select(1),
         )
 
-        assert conn.closed
+        assert not conn.closed
         assert not conn.invalidated
 
-    def test_noreconnect_rollback_plus_closewresult(self):
-        conn = self.db.connect(close_with_result=True)
+        conn.close()
+
+    def test_noreconnect_rollback(self):
+        # this test changes in 2.x due to autobegin.
+
+        conn = self.db.connect()
+
+        conn.execute(select(1))
 
         self.dbapi.shutdown("rollback_no_disconnect")
 
-        # raises error
-        with expect_warnings(
-            "An exception has occurred during handling .*"
-            "something broke on execute but we didn't lose the connection",
-            py2konly=True,
+        # previously, running a select() here which would fail would then
+        # trigger autorollback which would also fail, this is not the
+        # case now as autorollback does not normally occur
+        with expect_raises_message(
+            tsa.exc.DBAPIError,
+            r"something broke on rollback but we didn't lose the connection",
         ):
-            assert_raises_message(
-                tsa.exc.DBAPIError,
-                "something broke on rollback but we didn't "
-                "lose the connection",
-                conn.execute,
-                select(1),
-            )
+            conn.rollback()
 
-        assert conn.closed
+        assert not conn.closed
         assert not conn.invalidated
 
+        conn.close()
+
         assert_raises_message(
             tsa.exc.ResourceClosedError,
             "This Connection is closed",
@@ -696,50 +732,15 @@ class MockReconnectTest(fixtures.TestBase):
 
         self.dbapi.shutdown("rollback")
 
-        # raises error
-        with expect_warnings(
-            "An exception has occurred during handling .*"
-            "something broke on execute but we didn't lose the connection",
-            py2konly=True,
-        ):
-            assert_raises_message(
-                tsa.exc.DBAPIError,
-                "Lost the DB connection on rollback",
-                conn.execute,
-                select(1),
-            )
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "Lost the DB connection on rollback",
+            conn.rollback,
+        )
 
         assert not conn.closed
         assert conn.invalidated
 
-    def test_reconnect_on_reentrant_plus_closewresult(self):
-        conn = self.db.connect(close_with_result=True)
-
-        self.dbapi.shutdown("rollback")
-
-        # raises error
-        with expect_warnings(
-            "An exception has occurred during handling .*"
-            "something broke on execute but we didn't lose the connection",
-            py2konly=True,
-        ):
-            assert_raises_message(
-                tsa.exc.DBAPIError,
-                "Lost the DB connection on rollback",
-                conn.execute,
-                select(1),
-            )
-
-        assert conn.closed
-        assert not conn.invalidated
-
-        assert_raises_message(
-            tsa.exc.ResourceClosedError,
-            "This Connection is closed",
-            conn.execute,
-            select(1),
-        )
-
     def test_check_disconnect_no_cursor(self):
         conn = self.db.connect()
         result = conn.execute(select(1))
@@ -968,7 +969,6 @@ class CursorErrTest(fixtures.TestBase):
                 util.warn("Exception attempting to detect")
 
         eng.dialect._get_default_schema_name = get_default_schema_name
-        eng.dialect._check_unicode_description = mock.Mock()
         return eng
 
     def test_cursor_explode(self):
@@ -987,10 +987,8 @@ class CursorErrTest(fixtures.TestBase):
         assert_warns_message(
             exc.SAWarning, "Exception attempting to detect", db.connect
         )
-        # there's legacy py2k stuff happening here making this
-        # less smooth and probably buggy
         eq_(
-            db.pool.logger.error.mock_calls[0:1],
+            db.pool.logger.error.mock_calls,
             [call("Error closing cursor", exc_info=True)],
         )
 
@@ -1028,6 +1026,12 @@ class RealReconnectTest(fixtures.TestBase):
             assert conn.invalidated
 
             assert conn.invalidated
+
+            with expect_raises(tsa.exc.PendingRollbackError):
+                conn.execute(select(1))
+
+            conn.rollback()
+
             eq_(conn.execute(select(1)).scalar(), 1)
             assert not conn.invalidated
 
@@ -1036,9 +1040,28 @@ class RealReconnectTest(fixtures.TestBase):
             _assert_invalidated(conn.execute, select(1))
 
             assert conn.invalidated
+            conn.rollback()
+
             eq_(conn.execute(select(1)).scalar(), 1)
             assert not conn.invalidated
 
+    def test_detach_invalidated(self):
+        with self.engine.connect() as conn:
+            conn.invalidate()
+            with expect_raises_message(
+                exc.InvalidRequestError,
+                "Can't detach an invalidated Connection",
+            ):
+                conn.detach()
+
+    def test_detach_closed(self):
+        with self.engine.connect() as conn:
+            pass
+        with expect_raises_message(
+            exc.ResourceClosedError, "This Connection is closed"
+        ):
+            conn.detach()
+
     @testing.requires.independent_connections
     def test_multiple_invalidate(self):
         c1 = self.engine.connect()
@@ -1057,61 +1080,6 @@ class RealReconnectTest(fixtures.TestBase):
         # pool isn't replaced
         assert self.engine.pool is p2
 
-    def test_branched_invalidate_branch_to_parent(self):
-        with self.engine.connect() as c1:
-
-            with patch.object(self.engine.pool, "logger") as logger:
-                with testing.expect_deprecated_20(
-                    r"The Connection.connect\(\) method is considered legacy"
-                ):
-                    c1_branch = c1.connect()
-                eq_(c1_branch.execute(select(1)).scalar(), 1)
-
-                self.engine.test_shutdown()
-
-                _assert_invalidated(c1_branch.execute, select(1))
-                assert c1.invalidated
-                assert c1_branch.invalidated
-
-                c1_branch._revalidate_connection()
-                assert not c1.invalidated
-                assert not c1_branch.invalidated
-
-            assert "Invalidate connection" in logger.mock_calls[0][1][0]
-
-    def test_branched_invalidate_parent_to_branch(self):
-        with self.engine.connect() as c1:
-            with testing.expect_deprecated_20(
-                r"The Connection.connect\(\) method is considered legacy"
-            ):
-                c1_branch = c1.connect()
-            eq_(c1_branch.execute(select(1)).scalar(), 1)
-
-            self.engine.test_shutdown()
-
-            _assert_invalidated(c1.execute, select(1))
-            assert c1.invalidated
-            assert c1_branch.invalidated
-
-            c1._revalidate_connection()
-            assert not c1.invalidated
-            assert not c1_branch.invalidated
-
-    def test_branch_invalidate_state(self):
-        with self.engine.connect() as c1:
-            with testing.expect_deprecated_20(
-                r"The Connection.connect\(\) method is considered legacy"
-            ):
-                c1_branch = c1.connect()
-
-            eq_(c1_branch.execute(select(1)).scalar(), 1)
-
-            self.engine.test_shutdown()
-
-            _assert_invalidated(c1_branch.execute, select(1))
-            assert not c1_branch.closed
-            assert not c1_branch._still_open_and_dbapi_connection_is_valid
-
     def test_ensure_is_disconnect_gets_connection(self):
         def is_disconnect(e, conn, cursor):
             # connection is still present
@@ -1124,10 +1092,11 @@ class RealReconnectTest(fixtures.TestBase):
 
         with self.engine.connect() as conn:
             self.engine.test_shutdown()
-            with expect_warnings(
-                "An exception has occurred during handling .*", py2konly=True
-            ):
-                assert_raises(tsa.exc.DBAPIError, conn.execute, select(1))
+            assert_raises(tsa.exc.DBAPIError, conn.execute, select(1))
+
+            # aiosqlite is not able to run close() here without an
+            # error.
+            conn.invalidate()
 
     def test_rollback_on_invalid_plain(self):
         with self.engine.connect() as conn:
@@ -1148,8 +1117,23 @@ class RealReconnectTest(fixtures.TestBase):
             conn.begin()
             trans2 = conn.begin_nested()
             conn.invalidate()
+
+            # this passes silently, as it will often be involved
+            # in error catching schemes
             trans2.rollback()
 
+            # still invalid though
+            with expect_raises(exc.PendingRollbackError):
+                conn.begin_nested()
+
+    def test_no_begin_on_invalid(self):
+        with self.engine.connect() as conn:
+            conn.begin()
+            conn.invalidate()
+
+            with expect_raises(exc.PendingRollbackError):
+                conn.commit()
+
     def test_invalidate_twice(self):
         with self.engine.connect() as conn:
             conn.invalidate()
@@ -1193,6 +1177,7 @@ class RealReconnectTest(fixtures.TestBase):
             _assert_invalidated(conn.execute, select(1))
             assert not conn.closed
             assert conn.invalidated
+            conn.rollback()
             eq_(conn.execute(select(1)).scalar(), 1)
             assert not conn.invalidated
 
@@ -1375,10 +1360,6 @@ class InvalidateDuringResultTest(fixtures.TestBase):
             self.meta.drop_all(conn)
         self.engine.dispose()
 
-    @testing.crashes(
-        "oracle",
-        "cx_oracle 6 doesn't allow a close like this due to open cursors",
-    )
     @testing.fails_if(
         [
             "+mysqlconnector",
@@ -1390,6 +1371,7 @@ class InvalidateDuringResultTest(fixtures.TestBase):
             "+aiosqlite",
             "+aiomysql",
             "+asyncmy",
+            "+psycopg",
         ],
         "Buffers the result set and doesn't check for connection close",
     )
@@ -1422,9 +1404,7 @@ class ReconnectRecipeTest(fixtures.TestBase):
     # to get a real "cut the server off" kind of fixture we'd need to do
     # something in provisioning that seeks out the TCP connection at the
     # OS level and kills it.
-    __only_on__ = ("mysql+mysqldb", "mysql+pymysql")
-
-    future = False
+    __only_on__ = ("+mysqldb", "+pymysql")
 
     def make_engine(self, engine):
         num_retries = 3
@@ -1447,12 +1427,7 @@ class ReconnectRecipeTest(fixtures.TestBase):
                         )
                         connection.invalidate()
 
-                        if self.future:
-                            connection.rollback()
-                        else:
-                            trans = connection.get_transaction()
-                            if trans:
-                                trans.rollback()
+                        connection.rollback()
 
                         time.sleep(retry_interval)
                         context.cursor = (
@@ -1489,9 +1464,7 @@ class ReconnectRecipeTest(fixtures.TestBase):
     __backend__ = True
 
     def setup_test(self):
-        self.engine = engines.reconnecting_engine(
-            options=dict(future=self.future)
-        )
+        self.engine = engines.reconnecting_engine()
         self.meta = MetaData()
         self.table = Table(
             "sometable",
@@ -1576,12 +1549,4 @@ class ReconnectRecipeTest(fixtures.TestBase):
                     {"id": 6, "name": "some name 6"},
                 ],
             )
-            if self.future:
-                conn.rollback()
-            else:
-                trans = conn.get_transaction()
-                trans.rollback()
-
-
-class FutureReconnectRecipeTest(ReconnectRecipeTest):
-    future = True
+            conn.rollback()
