@@ -1,13 +1,16 @@
 import copy
+import dataclasses
 import pickle
 
 from sqlalchemy import event
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
+from sqlalchemy import inspect
 from sqlalchemy import Integer
+from sqlalchemy import JSON
+from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
-from sqlalchemy import util
 from sqlalchemy.ext.mutable import MutableComposite
 from sqlalchemy.ext.mutable import MutableDict
 from sqlalchemy.ext.mutable import MutableList
@@ -15,7 +18,8 @@ from sqlalchemy.ext.mutable import MutableSet
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import composite
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import declarative_base
+from sqlalchemy.orm import Session
 from sqlalchemy.orm.instrumentation import ClassManager
 from sqlalchemy.orm.mapper import Mapper
 from sqlalchemy.testing import assert_raises
@@ -23,6 +27,7 @@ from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
+from sqlalchemy.testing import is_true
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
@@ -41,7 +46,11 @@ class SubFoo(Foo):
     pass
 
 
-class FooWithEq(object):
+class Foo2(fixtures.BasicEntity):
+    pass
+
+
+class FooWithEq:
     def __init__(self, **kw):
         for k in kw:
             setattr(self, k, kw[k])
@@ -91,7 +100,45 @@ class MyPoint(Point):
         return value
 
 
-class _MutableDictTestFixture(object):
+@dataclasses.dataclass
+class DCPoint(MutableComposite):
+    x: int
+    y: int
+
+    def __setattr__(self, key, value):
+        object.__setattr__(self, key, value)
+        self.changed()
+
+    def __getstate__(self):
+        return self.x, self.y
+
+    def __setstate__(self, state):
+        self.x, self.y = state
+
+
+@dataclasses.dataclass
+class MyDCPoint(MutableComposite):
+    x: int
+    y: int
+
+    def __setattr__(self, key, value):
+        object.__setattr__(self, key, value)
+        self.changed()
+
+    def __getstate__(self):
+        return self.x, self.y
+
+    def __setstate__(self, state):
+        self.x, self.y = state
+
+    @classmethod
+    def coerce(cls, key, value):
+        if isinstance(value, tuple):
+            value = MyDCPoint(*value)
+        return value
+
+
+class _MutableDictTestFixture:
     @classmethod
     def _type_fixture(cls):
         return MutableDict
@@ -102,13 +149,108 @@ class _MutableDictTestFixture(object):
         ClassManager.dispatch._clear()
 
 
+class MiscTest(fixtures.TestBase):
+    @testing.combinations(True, False, argnames="pickleit")
+    def test_pickle_parent_multi_attrs(self, registry, connection, pickleit):
+        """test #8133"""
+
+        local_foo = Table(
+            "lf",
+            registry.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("j1", MutableDict.as_mutable(PickleType)),
+            Column("j2", MutableDict.as_mutable(PickleType)),
+            Column("j3", MutableDict.as_mutable(PickleType)),
+            Column("j4", MutableDict.as_mutable(PickleType)),
+        )
+
+        registry.map_imperatively(Foo2, local_foo)
+        registry.metadata.create_all(connection)
+
+        with Session(connection) as sess:
+
+            data = dict(
+                j1={"a": 1},
+                j2={"b": 2},
+                j3={"c": 3},
+                j4={"d": 4},
+            )
+            lf = Foo2(**data)
+            sess.add(lf)
+            sess.commit()
+
+        all_attrs = {"j1", "j2", "j3", "j4"}
+        for attr in all_attrs:
+            for loads, dumps in picklers():
+                with Session(connection) as sess:
+                    f1 = sess.scalars(select(Foo2)).first()
+                    if pickleit:
+                        f2 = loads(dumps(f1))
+                    else:
+                        f2 = f1
+
+                existing_dict = getattr(f2, attr)
+                existing_dict["q"] = "c"
+                eq_(
+                    inspect(f2).attrs[attr].history,
+                    ([existing_dict], (), ()),
+                )
+                for other_attr in all_attrs.difference([attr]):
+                    a = inspect(f2).attrs[other_attr].history
+                    b = ((), [data[other_attr]], ())
+                    eq_(a, b)
+
+    @testing.combinations("key_present", "key_non_present", argnames="present")
+    @testing.combinations(
+        ("transient", True),
+        ("detached", True),
+        ("detached", False),
+        argnames="merge_subject, load",
+    )
+    @testing.requires.json_type
+    def test_session_merge(
+        self, decl_base, connection, present, load, merge_subject
+    ):
+        """test #8446"""
+
+        class Thing(decl_base):
+            __tablename__ = "thing"
+            id = Column(Integer, primary_key=True)
+            data = Column(MutableDict.as_mutable(JSON))
+
+        decl_base.metadata.create_all(connection)
+
+        with Session(connection) as sess:
+            sess.add(Thing(id=1, data={"foo": "bar"}))
+            sess.commit()
+
+        if merge_subject == "transient":
+            t1_to_merge = Thing(id=1, data={"foo": "bar"})
+        elif merge_subject == "detached":
+            with Session(connection) as sess:
+                t1_to_merge = sess.get(Thing, 1)
+
+        with Session(connection) as sess:
+            already_present = None
+            if present == "key_present":
+                already_present = sess.get(Thing, 1)
+
+            t1_merged = sess.merge(t1_to_merge, load=load)
+
+            t1_merged.data["foo"] = "bat"
+            if present == "key_present":
+                is_(t1_merged, already_present)
+
+            is_true(inspect(t1_merged).attrs.data.history.added)
+
+
 class _MutableDictTestBase(_MutableDictTestFixture):
     run_define_tables = "each"
 
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
         sess = fixture_session()
@@ -122,7 +264,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
             ValueError,
             "Attribute 'data' does not accept objects of type",
             Foo,
-            data=set([1, 2, 3]),
+            data={1, 2, 3},
         )
 
     def test_in_place_mutation(self):
@@ -148,7 +290,10 @@ class _MutableDictTestBase(_MutableDictTestFixture):
             canary.mock_calls,
             [
                 mock.call(
-                    f1, attributes.Event(Foo.data.impl, attributes.OP_MODIFIED)
+                    f1,
+                    attributes.AttributeEventToken(
+                        Foo.data.impl, attributes.OP_MODIFIED
+                    ),
                 )
             ],
         )
@@ -312,7 +457,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"a": "b"})
 
 
-class _MutableListTestFixture(object):
+class _MutableListTestFixture:
     @classmethod
     def _type_fixture(cls):
         return MutableList
@@ -329,7 +474,7 @@ class _MutableListTestBase(_MutableListTestFixture):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
         sess = fixture_session()
@@ -343,7 +488,7 @@ class _MutableListTestBase(_MutableListTestFixture):
             ValueError,
             "Attribute 'data' does not accept objects of type",
             Foo,
-            data=set([1, 2, 3]),
+            data={1, 2, 3},
         )
 
     def test_in_place_mutation(self):
@@ -573,44 +718,24 @@ class _MutableListTestBase(_MutableListTestFixture):
         # print(repr(pickles))
         # return
 
-        if util.py3k:
-            pickles = [
-                b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
-                b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
-                b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
-                b"ccopy_reg\n_reconstructor\np0\n(csqlalchemy.ext.mutable\n"
-                b"MutableList\np1\nc__builtin__\nlist\np2\n(lp3\nI1\naI2\n"
-                b"atp4\nRp5\n(lp6\nI1\naI2\nab.",
-                b"ccopy_reg\n_reconstructor\nq\x00(csqlalchemy.ext.mutable\n"
-                b"MutableList\nq\x01c__builtin__\nlist\nq\x02]q\x03(K\x01K"
-                b"\x02etq\x04Rq\x05]q\x06(K\x01K\x02eb.",
-                b"\x80\x02csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
-                b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
-                b"\x80\x03csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
-                b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
-                b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
-                b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
-                b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
-            ]
-        else:
-            pickles = [
-                "\x80\x02csqlalchemy.ext.mutable\nMutableList\nq\x00]q\x01"
-                "(K\x01K\x02e\x85q\x02Rq\x03.",
-                "\x80\x02csqlalchemy.ext.mutable\nMutableList"
-                "\nq\x00]q\x01(K\x01K\x02e\x85q\x02Rq\x03.",
-                "csqlalchemy.ext.mutable\nMutableList\np0\n"
-                "((lp1\nI1\naI2\natp2\nRp3\n.",
-                "csqlalchemy.ext.mutable\nMutableList\nq\x00(]"
-                "q\x01(K\x01K\x02etq\x02Rq\x03.",
-                "\x80\x02csqlalchemy.ext.mutable\nMutableList"
-                "\nq\x01]q\x02(K\x01K\x02e\x85Rq\x03.",
-                "\x80\x02csqlalchemy.ext.mutable\nMutableList\n"
-                "q\x01]q\x02(K\x01K\x02e\x85Rq\x03.",
-                "csqlalchemy.ext.mutable\nMutableList\np1\n"
-                "((lp2\nI1\naI2\natRp3\n.",
-                "csqlalchemy.ext.mutable\nMutableList\nq\x01"
-                "(]q\x02(K\x01K\x02etRq\x03.",
-            ]
+        pickles = [
+            b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
+            b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
+            b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
+            b"ccopy_reg\n_reconstructor\np0\n(csqlalchemy.ext.mutable\n"
+            b"MutableList\np1\nc__builtin__\nlist\np2\n(lp3\nI1\naI2\n"
+            b"atp4\nRp5\n(lp6\nI1\naI2\nab.",
+            b"ccopy_reg\n_reconstructor\nq\x00(csqlalchemy.ext.mutable\n"
+            b"MutableList\nq\x01c__builtin__\nlist\nq\x02]q\x03(K\x01K"
+            b"\x02etq\x04Rq\x05]q\x06(K\x01K\x02eb.",
+            b"\x80\x02csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
+            b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
+            b"\x80\x03csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
+            b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
+            b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
+            b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
+            b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
+        ]
 
         for pickle_ in pickles:
             obj = pickle.loads(pickle_)
@@ -618,7 +743,7 @@ class _MutableListTestBase(_MutableListTestFixture):
             assert isinstance(obj, MutableList)
 
 
-class _MutableSetTestFixture(object):
+class _MutableSetTestFixture:
     @classmethod
     def _type_fixture(cls):
         return MutableSet
@@ -635,7 +760,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
         sess = fixture_session()
@@ -655,7 +780,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
     def test_clear(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
@@ -667,7 +792,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
     def test_pop(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1]))
+        f1 = Foo(data={1})
         sess.add(f1)
         sess.commit()
 
@@ -681,144 +806,144 @@ class _MutableSetTestBase(_MutableSetTestFixture):
     def test_add(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
         f1.data.add(5)
         sess.commit()
 
-        eq_(f1.data, set([1, 2, 5]))
+        eq_(f1.data, {1, 2, 5})
 
     def test_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data.update(set([2, 5]))
+        f1.data.update({2, 5})
         sess.commit()
 
-        eq_(f1.data, set([1, 2, 5]))
+        eq_(f1.data, {1, 2, 5})
 
     def test_binary_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data |= set([2, 5])
+        f1.data |= {2, 5}
         sess.commit()
 
-        eq_(f1.data, set([1, 2, 5]))
+        eq_(f1.data, {1, 2, 5})
 
     def test_intersection_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data.intersection_update(set([2, 5]))
+        f1.data.intersection_update({2, 5})
         sess.commit()
 
-        eq_(f1.data, set([2]))
+        eq_(f1.data, {2})
 
     def test_binary_intersection_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data &= set([2, 5])
+        f1.data &= {2, 5}
         sess.commit()
 
-        eq_(f1.data, set([2]))
+        eq_(f1.data, {2})
 
     def test_difference_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data.difference_update(set([2, 5]))
+        f1.data.difference_update({2, 5})
         sess.commit()
 
-        eq_(f1.data, set([1]))
+        eq_(f1.data, {1})
 
     def test_operator_difference_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data -= set([2, 5])
+        f1.data -= {2, 5}
         sess.commit()
 
-        eq_(f1.data, set([1]))
+        eq_(f1.data, {1})
 
     def test_symmetric_difference_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data.symmetric_difference_update(set([2, 5]))
+        f1.data.symmetric_difference_update({2, 5})
         sess.commit()
 
-        eq_(f1.data, set([1, 5]))
+        eq_(f1.data, {1, 5})
 
     def test_binary_symmetric_difference_update(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
 
-        f1.data ^= set([2, 5])
+        f1.data ^= {2, 5}
         sess.commit()
 
-        eq_(f1.data, set([1, 5]))
+        eq_(f1.data, {1, 5})
 
     def test_remove(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2, 3]))
+        f1 = Foo(data={1, 2, 3})
         sess.add(f1)
         sess.commit()
 
         f1.data.remove(2)
         sess.commit()
 
-        eq_(f1.data, set([1, 3]))
+        eq_(f1.data, {1, 3})
 
     def test_discard(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2, 3]))
+        f1 = Foo(data={1, 2, 3})
         sess.add(f1)
         sess.commit()
 
         f1.data.discard(2)
         sess.commit()
 
-        eq_(f1.data, set([1, 3]))
+        eq_(f1.data, {1, 3})
 
         f1.data.discard(2)
         sess.commit()
 
-        eq_(f1.data, set([1, 3]))
+        eq_(f1.data, {1, 3})
 
     def test_pickle_parent(self):
         sess = fixture_session()
 
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         sess.add(f1)
         sess.commit()
         f1.data
@@ -833,27 +958,27 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
     def test_unrelated_flush(self):
         sess = fixture_session()
-        f1 = Foo(data=set([1, 2]), unrelated_data="unrelated")
+        f1 = Foo(data={1, 2}, unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
         f1.unrelated_data = "unrelated 2"
         sess.flush()
         f1.data.add(3)
         sess.commit()
-        eq_(f1.data, set([1, 2, 3]))
+        eq_(f1.data, {1, 2, 3})
 
     def test_copy(self):
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         f1.data = copy.copy(f1.data)
-        eq_(f1.data, set([1, 2]))
+        eq_(f1.data, {1, 2})
 
     def test_deepcopy(self):
-        f1 = Foo(data=set([1, 2]))
+        f1 = Foo(data={1, 2})
         f1.data = copy.deepcopy(f1.data)
-        eq_(f1.data, set([1, 2]))
+        eq_(f1.data, {1, 2})
 
 
-class _MutableNoHashFixture(object):
+class _MutableNoHashFixture:
     @testing.fixture(autouse=True, scope="class")
     def set_class(self):
         global Foo
@@ -925,7 +1050,7 @@ class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_evt_on_flush_refresh(self):
         # test for #3427
@@ -970,6 +1095,7 @@ class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1003,10 +1129,10 @@ class MutableColumnCopyJSONTest(_MutableDictTestBase, fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         import json
-        from sqlalchemy.ext.declarative import declarative_base
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1050,14 +1176,13 @@ class MutableColumnCopyArrayTest(_MutableListTestBase, fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        from sqlalchemy.ext.declarative import declarative_base
         from sqlalchemy.sql.sqltypes import ARRAY
 
         MutableList = cls._type_fixture()
 
         Base = declarative_base(metadata=metadata)
 
-        class Mixin(object):
+        class Mixin:
             data = Column(MutableList.as_mutable(ARRAY(Integer)))
 
         class Foo(Mixin, Base):
@@ -1132,8 +1257,8 @@ class MutableAssocWithAttrInheritTest(
         foo = cls.tables.foo
         subfoo = cls.tables.subfoo
 
-        mapper(Foo, foo)
-        mapper(SubFoo, subfoo, inherits=Foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
+        cls.mapper_registry.map_imperatively(SubFoo, subfoo, inherits=Foo)
         MutableDict.associate_with_attribute(Foo.data)
 
     def test_in_place_mutation(self):
@@ -1188,6 +1313,7 @@ class MutableAssociationScalarJSONTest(
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1223,9 +1349,7 @@ class CustomMutableAssociationScalarJSONTest(
     @classmethod
     def _type_fixture(cls):
         if not (getattr(cls, "CustomMutableDict")):
-            MutableDict = super(
-                CustomMutableAssociationScalarJSONTest, cls
-            )._type_fixture()
+            MutableDict = super()._type_fixture()
 
             class CustomMutableDict(MutableDict):
                 pass
@@ -1239,6 +1363,7 @@ class CustomMutableAssociationScalarJSONTest(
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1277,7 +1402,7 @@ class CustomMutableAssociationScalarJSONTest(
         eq_(type(f1.data), self._type_fixture())
 
 
-class _CompositeTestBase(object):
+class _CompositeTestBase:
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -1329,7 +1454,7 @@ class MutableCompositeColumnDefaultTest(
 
         cls.Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo,
             foo,
             properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
@@ -1349,6 +1474,13 @@ class MutableCompositeColumnDefaultTest(
         assert f1 in sess.dirty
 
 
+class MutableDCCompositeColumnDefaultTest(MutableCompositeColumnDefaultTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        return DCPoint
+
+
 class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
     @classmethod
     def setup_mappers(cls):
@@ -1356,7 +1488,7 @@ class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
 
         cls.Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             FooWithEq,
             foo,
             properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
@@ -1368,20 +1500,29 @@ class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
             loads(dumps(u1))
 
 
+class MutableDCCompositesUnpickleTest(MutableCompositesUnpickleTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        return DCPoint
+
+
 class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
     @classmethod
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        Point = cls._type_fixture()
+        cls.Point = cls._type_fixture()
 
-        mapper(
-            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        cls.mapper_registry.map_imperatively(
+            Foo,
+            foo,
+            properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
         )
 
     def test_in_place_mutation(self):
         sess = fixture_session()
-        d = Point(3, 4)
+        d = self.Point(3, 4)
         f1 = Foo(data=d)
         sess.add(f1)
         sess.commit()
@@ -1389,11 +1530,11 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         f1.data.y = 5
         sess.commit()
 
-        eq_(f1.data, Point(3, 5))
+        eq_(f1.data, self.Point(3, 5))
 
     def test_pickle_of_parent(self):
         sess = fixture_session()
-        d = Point(3, 4)
+        d = self.Point(3, 4)
         f1 = Foo(data=d)
         sess.add(f1)
         sess.commit()
@@ -1414,11 +1555,11 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         f1 = Foo(data=None)
         sess.add(f1)
         sess.commit()
-        eq_(f1.data, Point(None, None))
+        eq_(f1.data, self.Point(None, None))
 
         f1.data.y = 5
         sess.commit()
-        eq_(f1.data, Point(None, 5))
+        eq_(f1.data, self.Point(None, 5))
 
     def test_set_illegal(self):
         f1 = Foo()
@@ -1433,7 +1574,7 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
     def test_unrelated_flush(self):
         sess = fixture_session()
-        f1 = Foo(data=Point(3, 4), unrelated_data="unrelated")
+        f1 = Foo(data=self.Point(3, 4), unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
         f1.unrelated_data = "unrelated 2"
@@ -1445,7 +1586,7 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
     def test_dont_reset_on_attr_refresh(self):
         sess = fixture_session()
-        f1 = Foo(data=Point(3, 4), unrelated_data="unrelated")
+        f1 = Foo(data=self.Point(3, 4), unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
 
@@ -1477,6 +1618,13 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         eq_(f1.data.y, 15)
 
 
+class MutableDCCompositesTest(MutableCompositesTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        return DCPoint
+
+
 class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
     @classmethod
     def setup_mappers(cls):
@@ -1486,7 +1634,7 @@ class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
 
         # in this case, this is not actually a MutableComposite.
         # so we don't expect it to track changes
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo,
             foo,
             properties={
@@ -1518,16 +1666,18 @@ class MutableCompositeCustomCoerceTest(
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        Point = cls._type_fixture()
+        cls.Point = cls._type_fixture()
 
-        mapper(
-            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        cls.mapper_registry.map_imperatively(
+            Foo,
+            foo,
+            properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
         )
 
     def test_custom_coerce(self):
         f = Foo()
         f.data = (3, 4)
-        eq_(f.data, Point(3, 4))
+        eq_(f.data, self.Point(3, 4))
 
     def test_round_trip_ok(self):
         sess = fixture_session()
@@ -1537,7 +1687,14 @@ class MutableCompositeCustomCoerceTest(
         sess.add(f)
         sess.commit()
 
-        eq_(f.data, Point(3, 4))
+        eq_(f.data, self.Point(3, 4))
+
+
+class MutableDCCompositeCustomCoerceTest(MutableCompositeCustomCoerceTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        return MyDCPoint
 
 
 class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
@@ -1563,16 +1720,18 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         foo = cls.tables.foo
         subfoo = cls.tables.subfoo
 
-        Point = cls._type_fixture()
+        cls.Point = cls._type_fixture()
 
-        mapper(
-            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        cls.mapper_registry.map_imperatively(
+            Foo,
+            foo,
+            properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
         )
-        mapper(SubFoo, subfoo, inherits=Foo)
+        cls.mapper_registry.map_imperatively(SubFoo, subfoo, inherits=Foo)
 
     def test_in_place_mutation_subclass(self):
         sess = fixture_session()
-        d = Point(3, 4)
+        d = self.Point(3, 4)
         f1 = SubFoo(data=d)
         sess.add(f1)
         sess.commit()
@@ -1580,11 +1739,11 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         f1.data.y = 5
         sess.commit()
 
-        eq_(f1.data, Point(3, 5))
+        eq_(f1.data, self.Point(3, 5))
 
     def test_pickle_of_parent_subclass(self):
         sess = fixture_session()
-        d = Point(3, 4)
+        d = self.Point(3, 4)
         f1 = SubFoo(data=d)
         sess.add(f1)
         sess.commit()
@@ -1599,3 +1758,10 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
             sess.add(f2)
             f2.data.y = 12
             assert f2 in sess.dirty
+
+
+class MutableInheritedDCCompositesTest(MutableInheritedCompositesTest):
+    @classmethod
+    def _type_fixture(cls):
+
+        return DCPoint
