@@ -30,6 +30,7 @@ from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import composite
 from sqlalchemy.orm import DeclarativeBase
+from sqlalchemy.orm import declared_attr
 from sqlalchemy.orm import deferred
 from sqlalchemy.orm import interfaces
 from sqlalchemy.orm import Mapped
@@ -629,6 +630,23 @@ class DCTransformsTest(AssertsCompiledSQL, fixtures.TestBase):
         b1 = Bar(mixin_value=5)
         eq_(b1.bar_value, 78)
 
+    def test_mixing_MappedAsDataclass_with_decorator_raises(self, registry):
+        """test #9211"""
+
+        class Mixin(MappedAsDataclass):
+            id: Mapped[int] = mapped_column(primary_key=True, init=False)
+
+        with expect_raises_message(
+            exc.InvalidRequestError,
+            "Class .*Foo.* is already a dataclass; ensure that "
+            "base classes / decorator styles of establishing dataclasses "
+            "are not being mixed. ",
+        ):
+
+            @registry.mapped_as_dataclass
+            class Foo(Mixin):
+                bar_value: Mapped[float] = mapped_column(default=78)
+
 
 class RelationshipDefaultFactoryTest(fixtures.TestBase):
     def test_list(self, dc_decl_base: Type[MappedAsDataclass]):
@@ -823,6 +841,291 @@ class RelationshipDefaultFactoryTest(fixtures.TestBase):
                 )
 
 
+class DataclassesForNonMappedClassesTest(fixtures.TestBase):
+    """test for cases added in #9179"""
+
+    def test_base_is_dc(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int
+
+        class Child(Parent):
+            __tablename__ = "child"
+            b: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_regex(repr(Child(5, 6)), r".*\.Child\(a=5, b=6\)")
+
+    def test_base_is_dc_plus_options(self):
+        class Parent(MappedAsDataclass, DeclarativeBase, unsafe_hash=True):
+            a: int
+
+        class Child(Parent, repr=False):
+            __tablename__ = "child"
+            b: Mapped[int] = mapped_column(primary_key=True)
+
+        c1 = Child(5, 6)
+        eq_(hash(c1), hash(Child(5, 6)))
+
+        # still reprs, because base has a repr, but b not included
+        eq_regex(repr(c1), r".*\.Child\(a=5\)")
+
+    def test_base_is_dc_init_var(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: InitVar[int]
+
+        class Child(Parent):
+            __tablename__ = "child"
+            b: Mapped[int] = mapped_column(primary_key=True)
+
+        c1 = Child(a=5, b=6)
+        eq_regex(repr(c1), r".*\.Child\(b=6\)")
+
+    def test_base_is_dc_field(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int = dataclasses.field(default=10)
+
+        class Child(Parent):
+            __tablename__ = "child"
+            b: Mapped[int] = mapped_column(primary_key=True, default=7)
+
+        c1 = Child(a=5, b=6)
+        eq_regex(repr(c1), r".*\.Child\(a=5, b=6\)")
+
+        c1 = Child(b=6)
+        eq_regex(repr(c1), r".*\.Child\(a=10, b=6\)")
+
+        c1 = Child()
+        eq_regex(repr(c1), r".*\.Child\(a=10, b=7\)")
+
+    def test_abstract_and_base_is_dc(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int
+
+        class Mixin(Parent):
+            __abstract__ = True
+            b: int
+
+        class Child(Mixin):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_regex(repr(Child(5, 6, 7)), r".*\.Child\(a=5, b=6, c=7\)")
+
+    def test_abstract_and_base_is_dc_plus_options(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int
+
+        class Mixin(Parent, unsafe_hash=True):
+            __abstract__ = True
+            b: int
+
+        class Child(Mixin, repr=False):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_(hash(Child(5, 6, 7)), hash(Child(5, 6, 7)))
+
+        eq_regex(repr(Child(5, 6, 7)), r".*\.Child\(a=5, b=6\)")
+
+    def test_abstract_and_base_is_dc_init_var(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: InitVar[int]
+
+        class Mixin(Parent):
+            __abstract__ = True
+            b: InitVar[int]
+
+        class Child(Mixin):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        c1 = Child(a=5, b=6, c=7)
+        eq_regex(repr(c1), r".*\.Child\(c=7\)")
+
+    def test_abstract_and_base_is_dc_field(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int = dataclasses.field(default=10)
+
+        class Mixin(Parent):
+            __abstract__ = True
+            b: int = dataclasses.field(default=7)
+
+        class Child(Mixin):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True, default=9)
+
+        c1 = Child(b=6, c=7)
+        eq_regex(repr(c1), r".*\.Child\(a=10, b=6, c=7\)")
+
+        c1 = Child()
+        eq_regex(repr(c1), r".*\.Child\(a=10, b=7, c=9\)")
+
+    def test_abstract_is_dc(self):
+        class Parent(DeclarativeBase):
+            a: int
+
+        class Mixin(MappedAsDataclass, Parent):
+            __abstract__ = True
+            b: int
+
+        class Child(Mixin):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_regex(repr(Child(6, 7)), r".*\.Child\(b=6, c=7\)")
+
+    def test_mixin_and_base_is_dc(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: int
+
+        @dataclasses.dataclass
+        class Mixin:
+            b: int
+
+        class Child(Mixin, Parent):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_regex(repr(Child(5, 6, 7)), r".*\.Child\(a=5, b=6, c=7\)")
+
+    def test_mixin_and_base_is_dc_init_var(self):
+        class Parent(MappedAsDataclass, DeclarativeBase):
+            a: InitVar[int]
+
+        @dataclasses.dataclass
+        class Mixin:
+            b: InitVar[int]
+
+        class Child(Mixin, Parent):
+            __tablename__ = "child"
+            c: Mapped[int] = mapped_column(primary_key=True)
+
+        eq_regex(repr(Child(a=5, b=6, c=7)), r".*\.Child\(c=7\)")
+
+    @testing.variation(
+        "dataclass_scope",
+        ["on_base", "on_mixin", "on_base_class", "on_sub_class"],
+    )
+    def test_mixin_w_inheritance(self, dataclass_scope):
+        """test #9226"""
+
+        if dataclass_scope.on_base:
+
+            class Base(DeclarativeBase, MappedAsDataclass):
+                pass
+
+        else:
+
+            class Base(DeclarativeBase):
+                pass
+
+        if dataclass_scope.on_mixin:
+
+            class Mixin(MappedAsDataclass):
+                @declared_attr.directive
+                @classmethod
+                def __tablename__(cls) -> str:
+                    return cls.__name__.lower()
+
+                @declared_attr.directive
+                @classmethod
+                def __mapper_args__(cls) -> Dict[str, Any]:
+                    return {
+                        "polymorphic_identity": cls.__name__,
+                        "polymorphic_on": "polymorphic_type",
+                    }
+
+                @declared_attr
+                @classmethod
+                def polymorphic_type(cls) -> Mapped[str]:
+                    return mapped_column(
+                        String,
+                        insert_default=cls.__name__,
+                        init=False,
+                    )
+
+        else:
+
+            class Mixin:
+                @declared_attr.directive
+                @classmethod
+                def __tablename__(cls) -> str:
+                    return cls.__name__.lower()
+
+                @declared_attr.directive
+                @classmethod
+                def __mapper_args__(cls) -> Dict[str, Any]:
+                    return {
+                        "polymorphic_identity": cls.__name__,
+                        "polymorphic_on": "polymorphic_type",
+                    }
+
+                if dataclass_scope.on_base or dataclass_scope.on_base_class:
+
+                    @declared_attr
+                    @classmethod
+                    def polymorphic_type(cls) -> Mapped[str]:
+                        return mapped_column(
+                            String,
+                            insert_default=cls.__name__,
+                            init=False,
+                        )
+
+                else:
+
+                    @declared_attr
+                    @classmethod
+                    def polymorphic_type(cls) -> Mapped[str]:
+                        return mapped_column(
+                            String,
+                            insert_default=cls.__name__,
+                        )
+
+        if dataclass_scope.on_base_class:
+
+            class Book(Mixin, MappedAsDataclass, Base):
+                id: Mapped[int] = mapped_column(
+                    Integer,
+                    primary_key=True,
+                    init=False,
+                )
+
+        else:
+
+            class Book(Mixin, Base):
+                if not dataclass_scope.on_sub_class:
+                    id: Mapped[int] = mapped_column(  # noqa: A001
+                        Integer, primary_key=True, init=False
+                    )
+                else:
+                    id: Mapped[int] = mapped_column(  # noqa: A001
+                        Integer,
+                        primary_key=True,
+                    )
+
+        if dataclass_scope.on_sub_class:
+
+            class Novel(MappedAsDataclass, Book):
+                id: Mapped[int] = mapped_column(  # noqa: A001
+                    ForeignKey("book.id"),
+                    primary_key=True,
+                    init=False,
+                )
+                description: Mapped[Optional[str]]
+
+        else:
+
+            class Novel(Book):
+                id: Mapped[int] = mapped_column(
+                    ForeignKey("book.id"),
+                    primary_key=True,
+                    init=False,
+                )
+                description: Mapped[Optional[str]]
+
+        n1 = Novel("the description")
+        eq_(n1.description, "the description")
+
+
 class DataclassArgsTest(fixtures.TestBase):
     dc_arg_names = ("init", "repr", "eq", "order", "unsafe_hash")
     if compat.py310:
@@ -986,12 +1289,17 @@ class DataclassArgsTest(fixtures.TestBase):
             create("g", 10) >= create("b", 7)
 
     def _assert_repr(self, cls, create, dc_arguments):
+        assert "__repr__" in cls.__dict__
         a1 = create("some data", 12)
         eq_regex(repr(a1), r".*A\(id=None, data='some data', x=12\)")
 
     def _assert_not_repr(self, cls, create, dc_arguments):
-        a1 = create("some data", 12)
-        eq_regex(repr(a1), r"<.*A object at 0x.*>")
+        assert "__repr__" not in cls.__dict__
+
+        # if a superclass has __repr__, then we still get repr.
+        # so can't test this
+        # a1 = create("some data", 12)
+        # eq_regex(repr(a1), r"<.*A object at 0x.*>")
 
     def _assert_init(self, cls, create, dc_arguments):
         if not dc_arguments.get("kw_only", False):
