@@ -1,20 +1,26 @@
+import dataclasses
+import operator
+import random
+
 import sqlalchemy as sa
 from sqlalchemy import ForeignKey
+from sqlalchemy import insert
 from sqlalchemy import Integer
 from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy import update
 from sqlalchemy.orm import aliased
+from sqlalchemy.orm import Composite
 from sqlalchemy.orm import composite
-from sqlalchemy.orm import CompositeProperty
 from sqlalchemy.orm import configure_mappers
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
+from sqlalchemy.testing import expect_raises_message
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import is_
 from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
@@ -79,8 +85,10 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 if args:
                     self.start, self.end = args
 
-        mapper(Graph, graphs, properties={"edges": relationship(Edge)})
-        mapper(
+        cls.mapper_registry.map_imperatively(
+            Graph, graphs, properties={"edges": relationship(Edge)}
+        )
+        cls.mapper_registry.map_imperatively(
             Edge,
             edges,
             properties={
@@ -89,14 +97,14 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             },
         )
 
-    def _fixture(self, future=False):
+    def _fixture(self):
         Graph, Edge, Point = (
             self.classes.Graph,
             self.classes.Edge,
             self.classes.Point,
         )
 
-        sess = Session(testing.db, future=future)
+        sess = Session(testing.db)
         g = Graph(
             id=1,
             edges=[
@@ -122,7 +130,7 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         g1 = sess.query(Graph).first()
         sess.close()
 
-        g = sess.query(Graph).get(g1.id)
+        g = sess.get(Graph, g1.id)
         eq_(
             [(e.start, e.end) for e in g.edges],
             [(Point(3, 4), Point(5, 6)), (Point(14, 5), Point(2, 7))],
@@ -141,7 +149,7 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         g.edges[1].end = Point(18, 4)
         sess.commit()
 
-        e = sess.query(Edge).get(g.edges[1].id)
+        e = sess.get(Edge, g.edges[1].id)
         eq_(e.end, Point(18, 4))
 
     def test_not_none(self):
@@ -177,8 +185,8 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         sess.close()
 
         def go():
-            g2 = (
-                sess.query(Graph).options(sa.orm.joinedload("edges")).get(g.id)
+            g2 = sess.get(
+                Graph, g.id, options=[sa.orm.joinedload(Graph.edges)]
             )
 
             eq_(
@@ -227,10 +235,10 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             is g.edges[1]
         )
 
-    def test_bulk_update_sql(self):
+    def test_update_crit_sql(self):
         Edge, Point = (self.classes.Edge, self.classes.Point)
 
-        sess = self._fixture(future=True)
+        sess = self._fixture()
 
         e1 = sess.execute(
             select(Edge).filter(Edge.start == Point(14, 5))
@@ -252,10 +260,10 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             dialect="default",
         )
 
-    def test_bulk_update_evaluate(self):
+    def test_update_crit_evaluate(self):
         Edge, Point = (self.classes.Edge, self.classes.Point)
 
-        sess = self._fixture(future=True)
+        sess = self._fixture()
 
         e1 = sess.execute(
             select(Edge).filter(Edge.start == Point(14, 5))
@@ -272,7 +280,16 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
 
         eq_(e1.end, Point(16, 10))
 
-    def test_bulk_update_fetch(self):
+        stmt = (
+            update(Edge)
+            .filter(Edge.start == Point(14, 5))
+            .values({Edge.end: Point(17, 8)})
+        )
+        sess.execute(stmt)
+
+        eq_(e1.end, Point(17, 8))
+
+    def test_update_crit_fetch(self):
         Edge, Point = (self.classes.Edge, self.classes.Point)
 
         sess = self._fixture()
@@ -286,6 +303,213 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
 
         eq_(e1.end, Point(16, 10))
 
+        q.update({Edge.end: Point(17, 8)}, synchronize_session="fetch")
+
+        eq_(e1.end, Point(17, 8))
+
+    @testing.combinations(
+        ("legacy",),
+        ("statement",),
+        ("values",),
+        ("stmt_returning", testing.requires.insertmanyvalues),
+        ("values_returning", testing.requires.insert_returning),
+    )
+    def test_bulk_insert(self, type_):
+        Edge, Point = (self.classes.Edge, self.classes.Point)
+        Graph = self.classes.Graph
+
+        sess = self._fixture()
+
+        graph = Graph(id=2)
+        sess.add(graph)
+        sess.flush()
+        graph_id = 2
+
+        data = [
+            {
+                "start": Point(random.randint(1, 50), random.randint(1, 50)),
+                "end": Point(random.randint(1, 50), random.randint(1, 50)),
+                "graph_id": graph_id,
+            }
+            for i in range(25)
+        ]
+        returning = False
+        if type_ == "statement":
+            sess.execute(insert(Edge), data)
+        elif type_ == "stmt_returning":
+            result = sess.scalars(insert(Edge).returning(Edge), data)
+            returning = True
+        elif type_ == "values":
+            sess.execute(insert(Edge).values(data))
+        elif type_ == "values_returning":
+            result = sess.scalars(insert(Edge).values(data).returning(Edge))
+            returning = True
+        elif type_ == "legacy":
+            sess.bulk_insert_mappings(Edge, data)
+        else:
+            assert False
+
+        if returning:
+            eq_(result.all(), [Edge(rec["start"], rec["end"]) for rec in data])
+
+        edges = self.tables.edges
+        eq_(
+            sess.execute(
+                select(edges.c["x1", "y1", "x2", "y2"])
+                .where(edges.c.graph_id == graph_id)
+                .order_by(edges.c.id)
+            ).all(),
+            [
+                (e["start"].x, e["start"].y, e["end"].x, e["end"].y)
+                for e in data
+            ],
+        )
+
+    @testing.combinations("legacy", "statement")
+    def test_bulk_insert_heterogeneous(self, type_):
+        Edge, Point = (self.classes.Edge, self.classes.Point)
+        Graph = self.classes.Graph
+
+        sess = self._fixture()
+
+        graph = Graph(id=2)
+        sess.add(graph)
+        sess.flush()
+        graph_id = 2
+
+        d1 = [
+            {
+                "start": Point(random.randint(1, 50), random.randint(1, 50)),
+                "end": Point(random.randint(1, 50), random.randint(1, 50)),
+                "graph_id": graph_id,
+            }
+            for i in range(3)
+        ]
+        d2 = [
+            {
+                "start": Point(random.randint(1, 50), random.randint(1, 50)),
+                "graph_id": graph_id,
+            }
+            for i in range(2)
+        ]
+        d3 = [
+            {
+                "x2": random.randint(1, 50),
+                "y2": random.randint(1, 50),
+                "graph_id": graph_id,
+            }
+            for i in range(2)
+        ]
+        data = d1 + d2 + d3
+        random.shuffle(data)
+
+        assert_data = [
+            {
+                "start": d["start"] if "start" in d else None,
+                "end": d["end"]
+                if "end" in d
+                else Point(d["x2"], d["y2"])
+                if "x2" in d
+                else None,
+                "graph_id": d["graph_id"],
+            }
+            for d in data
+        ]
+
+        if type_ == "statement":
+            sess.execute(insert(Edge), data)
+        elif type_ == "legacy":
+            sess.bulk_insert_mappings(Edge, data)
+        else:
+            assert False
+
+        edges = self.tables.edges
+        eq_(
+            sess.execute(
+                select(edges.c["x1", "y1", "x2", "y2"])
+                .where(edges.c.graph_id == graph_id)
+                .order_by(edges.c.id)
+            ).all(),
+            [
+                (
+                    e["start"].x if e["start"] else None,
+                    e["start"].y if e["start"] else None,
+                    e["end"].x if e["end"] else None,
+                    e["end"].y if e["end"] else None,
+                )
+                for e in assert_data
+            ],
+        )
+
+    @testing.combinations("legacy", "statement")
+    def test_bulk_update(self, type_):
+        Edge, Point = (self.classes.Edge, self.classes.Point)
+        Graph = self.classes.Graph
+
+        sess = self._fixture()
+
+        graph = Graph(id=2)
+        sess.add(graph)
+        sess.flush()
+        graph_id = 2
+
+        data = [
+            {
+                "start": Point(random.randint(1, 50), random.randint(1, 50)),
+                "end": Point(random.randint(1, 50), random.randint(1, 50)),
+                "graph_id": graph_id,
+            }
+            for i in range(25)
+        ]
+        sess.execute(insert(Edge), data)
+
+        inserted_data = [
+            dict(row._mapping)
+            for row in sess.execute(
+                select(Edge.id, Edge.start, Edge.end, Edge.graph_id)
+                .where(Edge.graph_id == graph_id)
+                .order_by(Edge.id)
+            )
+        ]
+
+        to_update = []
+        updated_pks = {}
+        for rec in random.choices(inserted_data, k=7):
+            rec_copy = dict(rec)
+            updated_pks[rec_copy["id"]] = rec_copy
+            rec_copy["start"] = Point(
+                random.randint(1, 50), random.randint(1, 50)
+            )
+            rec_copy["end"] = Point(
+                random.randint(1, 50), random.randint(1, 50)
+            )
+            to_update.append(rec_copy)
+
+        expected_dataset = [
+            updated_pks[row["id"]] if row["id"] in updated_pks else row
+            for row in inserted_data
+        ]
+
+        if type_ == "statement":
+            sess.execute(update(Edge), to_update)
+        elif type_ == "legacy":
+            sess.bulk_update_mappings(Edge, to_update)
+        else:
+            assert False
+
+        edges = self.tables.edges
+        eq_(
+            sess.execute(
+                select(edges.c["x1", "y1", "x2", "y2"])
+                .where(edges.c.graph_id == graph_id)
+                .order_by(edges.c.id)
+            ).all(),
+            [
+                (e["start"].x, e["start"].y, e["end"].x, e["end"].y)
+                for e in expected_dataset
+            ],
+        )
+
     def test_get_history(self):
         Edge = self.classes.Edge
         Point = self.classes.Point
@@ -377,7 +601,7 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         """test saving a null composite value
 
         See google groups thread for more context:
-        http://groups.google.com/group/sqlalchemy/browse_thread/thread/0c6580a1761b2c29
+        https://groups.google.com/group/sqlalchemy/browse_thread/thread/0c6580a1761b2c29
 
         """
 
@@ -391,7 +615,7 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         sess.add(g)
         sess.commit()
 
-        g2 = sess.query(Graph).get(1)
+        g2 = sess.get(Graph, 1)
         assert g2.edges[-1].start.x is None
         assert g2.edges[-1].start.y is None
 
@@ -411,6 +635,131 @@ class PointTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         e = Edge()
         eq_(e.start, None)
 
+    def test_no_name_declarative(self, decl_base, connection):
+        """test #7751"""
+
+        class Point:
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+
+            def __composite_values__(self):
+                return self.x, self.y
+
+            def __repr__(self):
+                return "Point(x=%r, y=%r)" % (self.x, self.y)
+
+            def __eq__(self, other):
+                return (
+                    isinstance(other, Point)
+                    and other.x == self.x
+                    and other.y == self.y
+                )
+
+            def __ne__(self, other):
+                return not self.__eq__(other)
+
+        class Vertex(decl_base):
+            __tablename__ = "vertices"
+
+            id = Column(Integer, primary_key=True)
+            x1 = Column(Integer)
+            y1 = Column(Integer)
+            x2 = Column(Integer)
+            y2 = Column(Integer)
+
+            start = composite(Point, x1, y1)
+            end = composite(Point, x2, y2)
+
+        self.assert_compile(
+            select(Vertex),
+            "SELECT vertices.id, vertices.x1, vertices.y1, vertices.x2, "
+            "vertices.y2 FROM vertices",
+        )
+
+        decl_base.metadata.create_all(connection)
+        s = Session(connection)
+        hv = Vertex(start=Point(1, 2), end=Point(3, 4))
+        s.add(hv)
+        s.commit()
+
+        is_(
+            hv,
+            s.scalars(
+                select(Vertex).where(Vertex.start == Point(1, 2))
+            ).first(),
+        )
+
+    def test_no_name_declarative_two(self, decl_base, connection):
+        """test #7752"""
+
+        class Point:
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+
+            def __composite_values__(self):
+                return self.x, self.y
+
+            def __repr__(self):
+                return "Point(x=%r, y=%r)" % (self.x, self.y)
+
+            def __eq__(self, other):
+                return (
+                    isinstance(other, Point)
+                    and other.x == self.x
+                    and other.y == self.y
+                )
+
+            def __ne__(self, other):
+                return not self.__eq__(other)
+
+        class Vertex:
+            def __init__(self, start, end):
+                self.start = start
+                self.end = end
+
+            @classmethod
+            def _generate(self, x1, y1, x2, y2):
+                """generate a Vertex from a row"""
+                return Vertex(Point(x1, y1), Point(x2, y2))
+
+            def __composite_values__(self):
+                return (
+                    self.start.__composite_values__()
+                    + self.end.__composite_values__()
+                )
+
+        class HasVertex(decl_base):
+            __tablename__ = "has_vertex"
+            id = Column(Integer, primary_key=True)
+            x1 = Column(Integer)
+            y1 = Column(Integer)
+            x2 = Column(Integer)
+            y2 = Column(Integer)
+
+            vertex = composite(Vertex._generate, x1, y1, x2, y2)
+
+        self.assert_compile(
+            select(HasVertex),
+            "SELECT has_vertex.id, has_vertex.x1, has_vertex.y1, "
+            "has_vertex.x2, has_vertex.y2 FROM has_vertex",
+        )
+
+        decl_base.metadata.create_all(connection)
+        s = Session(connection)
+        hv = HasVertex(vertex=Vertex(Point(1, 2), Point(3, 4)))
+        s.add(hv)
+        s.commit()
+        is_(
+            hv,
+            s.scalars(
+                select(HasVertex).where(
+                    HasVertex.vertex == Vertex(Point(1, 2), Point(3, 4))
+                )
+            ).first(),
+        )
+
 
 class NestedTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     @classmethod
@@ -428,7 +777,7 @@ class NestedTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         )
 
     def _fixture(self):
-        class AB(object):
+        class AB:
             def __init__(self, a, b, cd):
                 self.a = a
                 self.b = b
@@ -452,7 +801,7 @@ class NestedTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             def __ne__(self, other):
                 return not self.__eq__(other)
 
-        class CD(object):
+        class CD:
             def __init__(self, c, d):
                 self.c = c
                 self.d = d
@@ -470,12 +819,12 @@ class NestedTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             def __ne__(self, other):
                 return not self.__eq__(other)
 
-        class Thing(object):
+        class Thing:
             def __init__(self, ab):
                 self.ab = ab
 
         stuff = self.tables.stuff
-        mapper(
+        self.mapper_registry.map_imperatively(
             Thing,
             stuff,
             properties={
@@ -541,7 +890,7 @@ class PrimaryKeyTest(fixtures.MappedTest):
             def __init__(self, version):
                 self.version = version
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Graph,
             graphs,
             properties={
@@ -566,7 +915,7 @@ class PrimaryKeyTest(fixtures.MappedTest):
         sess = self._fixture()
         g = sess.query(Graph).first()
 
-        g2 = sess.query(Graph).get([g.id, g.version_id])
+        g2 = sess.get(Graph, [g.id, g.version_id])
         eq_(g.version, g2.version)
 
     def test_get_by_composite(self):
@@ -575,7 +924,7 @@ class PrimaryKeyTest(fixtures.MappedTest):
         sess = self._fixture()
         g = sess.query(Graph).first()
 
-        g2 = sess.query(Graph).get(Version(g.id, g.version_id))
+        g2 = sess.get(Graph, Version(g.id, g.version_id))
         eq_(g.version, g2.version)
 
     def test_pk_mutation(self):
@@ -587,7 +936,7 @@ class PrimaryKeyTest(fixtures.MappedTest):
 
         g.version = Version(2, 1)
         sess.commit()
-        g2 = sess.query(Graph).get(Version(2, 1))
+        g2 = sess.get(Graph, Version(2, 1))
         eq_(g.version, g2.version)
 
     @testing.fails_on_everything_except("sqlite")
@@ -605,6 +954,33 @@ class PrimaryKeyTest(fixtures.MappedTest):
         eq_(g.version, g2.version)
 
 
+class PrimaryKeyTestDataclasses(PrimaryKeyTest):
+    @classmethod
+    def setup_mappers(cls):
+        graphs = cls.tables.graphs
+
+        @dataclasses.dataclass
+        class Version:
+            id: int
+            version: int
+
+        cls.classes.Version = Version
+
+        class Graph(cls.Comparable):
+            def __init__(self, version):
+                self.version = version
+
+        cls.mapper_registry.map_imperatively(
+            Graph,
+            graphs,
+            properties={
+                "version": sa.orm.composite(
+                    Version, graphs.c.id, graphs.c.version_id
+                )
+            },
+        )
+
+
 class DefaultsTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -658,7 +1034,7 @@ class DefaultsTest(fixtures.MappedTest):
                     self.x4,
                 )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foobar,
             foobars,
             properties=dict(
@@ -754,7 +1130,7 @@ class MappedSelectTest(fixtures.MappedTest):
             .alias("descriptions_values")
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Descriptions,
             descriptions,
             properties={
@@ -765,7 +1141,7 @@ class MappedSelectTest(fixtures.MappedTest):
             },
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Values,
             desc_values,
             properties={
@@ -775,7 +1151,7 @@ class MappedSelectTest(fixtures.MappedTest):
             },
         )
 
-    def test_set_composite_attrs_via_selectable(self):
+    def test_set_composite_attrs_via_selectable(self, connection):
         Values, CustomValues, values, Descriptions, descriptions = (
             self.classes.Values,
             self.classes.CustomValues,
@@ -796,11 +1172,11 @@ class MappedSelectTest(fixtures.MappedTest):
         session.add(d)
         session.commit()
         eq_(
-            testing.db.execute(descriptions.select()).fetchall(),
+            connection.execute(descriptions.select()).fetchall(),
             [(1, "Color", "Number")],
         )
         eq_(
-            testing.db.execute(values.select()).fetchall(),
+            connection.execute(values.select()).fetchall(),
             [(1, 1, "Red", "5"), (2, 1, "Blue", "1")],
         )
 
@@ -851,12 +1227,12 @@ class ManyToOneTest(fixtures.MappedTest):
                     and other.b2 == self.b2
                 )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             A,
             a,
             properties={"b2": relationship(B), "c": composite(C, "b1", "b2")},
         )
-        mapper(B, b)
+        cls.mapper_registry.map_imperatively(B, b)
 
     def test_early_configure(self):
         # test [ticket:2935], that we can call a composite
@@ -955,7 +1331,7 @@ class ConfigurationTest(fixtures.MappedTest):
             self.classes.Point,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Edge,
             edge,
             properties={
@@ -973,7 +1349,7 @@ class ConfigurationTest(fixtures.MappedTest):
             self.classes.Point,
         )
 
-        m = mapper(Edge, edge)
+        m = self.mapper_registry.map_imperatively(Edge, edge)
         m.add_property("start", sa.orm.composite(Point, Edge.x1, Edge.y1))
         m.add_property("end", sa.orm.composite(Point, Edge.x2, Edge.y2))
 
@@ -986,7 +1362,7 @@ class ConfigurationTest(fixtures.MappedTest):
             self.classes.Point,
         )
 
-        m = mapper(Edge, edge)
+        m = self.mapper_registry.map_imperatively(Edge, edge)
         m.add_property("start", sa.orm.composite(Point, "x1", "y1"))
         m.add_property("end", sa.orm.composite(Point, "x2", "y2"))
 
@@ -998,7 +1374,7 @@ class ConfigurationTest(fixtures.MappedTest):
             self.classes.Edge,
             self.classes.Point,
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Edge,
             edge,
             properties={
@@ -1018,7 +1394,7 @@ class ConfigurationTest(fixtures.MappedTest):
             self.classes.Edge,
             self.classes.Point,
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Edge,
             edge,
             properties={
@@ -1091,14 +1467,14 @@ class ComparatorTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
 
         if custom:
 
-            class CustomComparator(sa.orm.CompositeProperty.Comparator):
+            class CustomComparator(sa.orm.Composite.Comparator):
                 def near(self, other, d):
                     clauses = self.__clause_element__().clauses
                     diff_x = clauses[0] - other.x
                     diff_y = clauses[1] - other.y
                     return diff_x * diff_x + diff_y * diff_y <= d * d
 
-            mapper(
+            self.mapper_registry.map_imperatively(
                 Edge,
                 edge,
                 properties={
@@ -1112,7 +1488,7 @@ class ComparatorTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 },
             )
         else:
-            mapper(
+            self.mapper_registry.map_imperatively(
                 Edge,
                 edge,
                 properties={
@@ -1121,35 +1497,76 @@ class ComparatorTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 },
             )
 
-    def test_comparator_behavior_default(self):
-        self._fixture(False)
-        self._test_comparator_behavior()
+    @testing.combinations(True, False, argnames="custom")
+    @testing.combinations(
+        (operator.lt, "<", ">"),
+        (operator.gt, ">", "<"),
+        (operator.eq, "=", "="),
+        (operator.ne, "!=", "!="),
+        (operator.le, "<=", ">="),
+        (operator.ge, ">=", "<="),
+        argnames="operator, fwd_op, rev_op",
+    )
+    def test_comparator_behavior(self, custom, operator, fwd_op, rev_op):
+        self._fixture(custom)
+        Edge, Point = self.classes("Edge", "Point")
 
-    def test_comparator_behavior_custom(self):
-        self._fixture(True)
-        self._test_comparator_behavior()
+        self.assert_compile(
+            select(Edge).filter(operator(Edge.start, Point(3, 4))),
+            "SELECT edge.id, edge.x1, edge.y1, edge.x2, edge.y2 FROM edge "
+            f"WHERE edge.x1 {fwd_op} :x1_1 AND edge.y1 {fwd_op} :y1_1",
+            checkparams={"x1_1": 3, "y1_1": 4},
+        )
 
-    def _test_comparator_behavior(self):
-        Edge, Point = (self.classes.Edge, self.classes.Point)
+        self.assert_compile(
+            select(Edge).filter(~operator(Edge.start, Point(3, 4))),
+            "SELECT edge.id, edge.x1, edge.y1, edge.x2, edge.y2 FROM edge "
+            f"WHERE NOT (edge.x1 {fwd_op} :x1_1 AND edge.y1 {fwd_op} :y1_1)",
+            checkparams={"x1_1": 3, "y1_1": 4},
+        )
 
-        sess = fixture_session()
-        e1 = Edge(Point(3, 4), Point(5, 6))
-        e2 = Edge(Point(14, 5), Point(2, 7))
-        sess.add_all([e1, e2])
-        sess.commit()
+    @testing.combinations(True, False, argnames="custom")
+    @testing.combinations(
+        (operator.lt, "<", ">"),
+        (operator.gt, ">", "<"),
+        (operator.eq, "=", "="),
+        (operator.ne, "!=", "!="),
+        (operator.le, "<=", ">="),
+        (operator.ge, ">=", "<="),
+        argnames="op, fwd_op, rev_op",
+    )
+    def test_comparator_null(self, custom, op, fwd_op, rev_op):
+        self._fixture(custom)
+        Edge, Point = self.classes("Edge", "Point")
 
-        assert sess.query(Edge).filter(Edge.start == Point(3, 4)).one() is e1
-
-        assert sess.query(Edge).filter(Edge.start != Point(3, 4)).first() is e2
-
-        eq_(sess.query(Edge).filter(Edge.start == None).all(), [])  # noqa
+        if op is operator.eq:
+            self.assert_compile(
+                select(Edge).filter(op(Edge.start, None)),
+                "SELECT edge.id, edge.x1, edge.y1, edge.x2, edge.y2 FROM edge "
+                "WHERE edge.x1 IS NULL AND edge.y1 IS NULL",
+                checkparams={},
+            )
+        elif op is operator.ne:
+            self.assert_compile(
+                select(Edge).filter(op(Edge.start, None)),
+                "SELECT edge.id, edge.x1, edge.y1, edge.x2, edge.y2 FROM edge "
+                "WHERE edge.x1 IS NOT NULL AND edge.y1 IS NOT NULL",
+                checkparams={},
+            )
+        else:
+            with expect_raises_message(
+                sa.exc.ArgumentError,
+                r"Only '=', '!=', .* operators can be used "
+                r"with None/True/False",
+            ):
+                select(Edge).filter(op(Edge.start, None))
 
     def test_default_comparator_factory(self):
         self._fixture(False)
         Edge = self.classes.Edge
         start_prop = Edge.start.property
 
-        assert start_prop.comparator_factory is CompositeProperty.Comparator
+        assert start_prop.comparator_factory is Composite.Comparator
 
     def test_custom_comparator_factory(self):
         self._fixture(True)
