@@ -16,6 +16,7 @@ from sqlalchemy.orm.util import identity_key
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import is_
+from sqlalchemy.testing.fixtures import fixture_session
 from test.orm import _fixtures
 
 
@@ -64,9 +65,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         assert not insp.is_aliased_class
 
     def test_mapper_selectable_fixed(self):
-        from sqlalchemy.orm import mapper
-
-        class Foo(object):
+        class Foo:
             pass
 
         class Bar(Foo):
@@ -74,8 +73,10 @@ class TestORMInspection(_fixtures.FixtureTest):
 
         user_table = self.tables.users
         addresses_table = self.tables.addresses
-        mapper(Foo, user_table, with_polymorphic=(Bar,))
-        mapper(
+        self.mapper_registry.map_imperatively(
+            Foo, user_table, with_polymorphic=(Bar,)
+        )
+        self.mapper_registry.map_imperatively(
             Bar,
             addresses_table,
             inherits=Foo,
@@ -95,7 +96,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         assert insp.is_aliased_class
 
     def test_not_mapped_class(self):
-        class Foo(object):
+        class Foo:
             pass
 
         assert_raises_message(
@@ -106,7 +107,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         )
 
     def test_not_mapped_instance(self):
-        class Foo(object):
+        class Foo:
             pass
 
         assert_raises_message(
@@ -168,7 +169,7 @@ class TestORMInspection(_fixtures.FixtureTest):
         rel = inspect(User).relationships
 
         eq_(rel.addresses, User.addresses.property)
-        eq_(set(rel.keys()), set(["orders", "addresses"]))
+        eq_(set(rel.keys()), {"orders", "addresses"})
 
     def test_insp_relationship_prop(self):
         User = self.classes.User
@@ -234,17 +235,15 @@ class TestORMInspection(_fixtures.FixtureTest):
     def test_extension_types(self):
         from sqlalchemy.ext.associationproxy import (
             association_proxy,
-            ASSOCIATION_PROXY,
+            AssociationProxyExtensionType,
         )
         from sqlalchemy.ext.hybrid import (
             hybrid_property,
             hybrid_method,
-            HYBRID_PROPERTY,
-            HYBRID_METHOD,
+            HybridExtensionType,
         )
         from sqlalchemy import Table, MetaData, Integer, Column
-        from sqlalchemy.orm import mapper
-        from sqlalchemy.orm.interfaces import NOT_EXTENSION
+        from sqlalchemy.orm.interfaces import NotExtension
 
         class SomeClass(self.classes.User):
             some_assoc = association_proxy("addresses", "email_address")
@@ -277,26 +276,28 @@ class TestORMInspection(_fixtures.FixtureTest):
             Column("id", Integer, primary_key=True),
             Column("s_id", ForeignKey("sometable.id")),
         )
-        mapper(SomeClass, t, properties={"addresses": relationship(Address)})
-        mapper(Address, ta)
-        mapper(SomeSubClass, inherits=SomeClass)
+        self.mapper_registry.map_imperatively(
+            SomeClass, t, properties={"addresses": relationship(Address)}
+        )
+        self.mapper_registry.map_imperatively(Address, ta)
+        self.mapper_registry.map_imperatively(SomeSubClass, inherits=SomeClass)
 
         insp = inspect(SomeSubClass)
         eq_(
-            dict(
-                (k, v.extension_type)
-                for k, v in list(insp.all_orm_descriptors.items())
-            ),
             {
-                "id": NOT_EXTENSION,
-                "name": NOT_EXTENSION,
-                "name_syn": NOT_EXTENSION,
-                "addresses": NOT_EXTENSION,
-                "orders": NOT_EXTENSION,
-                "upper_name": HYBRID_PROPERTY,
-                "foo": HYBRID_PROPERTY,
-                "conv": HYBRID_METHOD,
-                "some_assoc": ASSOCIATION_PROXY,
+                k: v.extension_type
+                for k, v in list(insp.all_orm_descriptors.items())
+            },
+            {
+                "id": NotExtension.NOT_EXTENSION,
+                "name": NotExtension.NOT_EXTENSION,
+                "name_syn": NotExtension.NOT_EXTENSION,
+                "addresses": NotExtension.NOT_EXTENSION,
+                "orders": NotExtension.NOT_EXTENSION,
+                "upper_name": HybridExtensionType.HYBRID_PROPERTY,
+                "foo": HybridExtensionType.HYBRID_PROPERTY,
+                "conv": HybridExtensionType.HYBRID_METHOD,
+                "some_assoc": AssociationProxyExtensionType.ASSOCIATION_PROXY,
             },
         )
         is_(
@@ -329,7 +330,7 @@ class TestORMInspection(_fixtures.FixtureTest):
 
         eq_(
             set(insp.attrs.keys()),
-            set(["id", "name", "name_syn", "addresses", "orders"]),
+            {"id", "name", "name_syn", "addresses", "orders"},
         )
         eq_(insp.attrs.name.value, "ed")
         eq_(insp.attrs.name.loaded_value, "ed")
@@ -353,29 +354,30 @@ class TestORMInspection(_fixtures.FixtureTest):
         eq_(insp.attrs.addresses.loaded_value, NO_VALUE)
         # regular accessor sets it
         eq_(insp.attrs.addresses.value, [])
-        # now the None is there
-        eq_(insp.attrs.addresses.loaded_value, [])
+        # stays as NO_VALUE, this is #4519
+        eq_(insp.attrs.addresses.loaded_value, NO_VALUE)
 
     def test_instance_state_collection_attr_hist(self):
         User = self.classes.User
         u1 = User(name="ed")
         insp = inspect(u1)
         hist = insp.attrs.addresses.history
-        eq_(hist.unchanged, None)
+        eq_(hist.unchanged, ())
         u1.addresses
         hist = insp.attrs.addresses.history
-        eq_(hist.unchanged, [])
+        # stays, this is #4519
+        eq_(hist.unchanged, ())
 
     def test_instance_state_scalar_attr_hist(self):
         User = self.classes.User
         u1 = User(name="ed")
-        sess = Session()
+        sess = fixture_session()
         sess.add(u1)
         sess.commit()
         assert "name" not in u1.__dict__
         insp = inspect(u1)
         hist = insp.attrs.name.history
-        eq_(hist.unchanged, None)
+        eq_(hist.unchanged, ())
         assert "name" not in u1.__dict__
 
     def test_instance_state_collection_attr_load_hist(self):
@@ -386,12 +388,13 @@ class TestORMInspection(_fixtures.FixtureTest):
         eq_(hist.unchanged, ())
         u1.addresses
         hist = insp.attrs.addresses.load_history()
-        eq_(hist.unchanged, [])
+        # stays, this is #4519
+        eq_(hist.unchanged, ())
 
     def test_instance_state_scalar_attr_hist_load(self):
         User = self.classes.User
         u1 = User(name="ed")
-        sess = Session()
+        sess = fixture_session()
         sess.add(u1)
         sess.commit()
         assert "name" not in u1.__dict__
@@ -404,19 +407,19 @@ class TestORMInspection(_fixtures.FixtureTest):
         class Thing(InspectionAttr):
             pass
 
-        class AnonClass(object):
+        class AnonClass:
             __foo__ = "bar"
             __bat__ = Thing()
 
-        from sqlalchemy.orm import mapper, column_property
+        from sqlalchemy.orm import column_property
         from sqlalchemy.ext.hybrid import hybrid_property
 
-        m = mapper(AnonClass, self.tables.users)
+        m = self.mapper_registry.map_imperatively(AnonClass, self.tables.users)
 
-        eq_(set(inspect(AnonClass).attrs.keys()), set(["id", "name"]))
+        eq_(set(inspect(AnonClass).attrs.keys()), {"id", "name"})
         eq_(
             set(inspect(AnonClass).all_orm_descriptors.keys()),
-            set(["id", "name"]),
+            {"id", "name"},
         )
 
         m.add_property("q", column_property(self.tables.users.c.name))
@@ -426,10 +429,10 @@ class TestORMInspection(_fixtures.FixtureTest):
 
         AnonClass.foob = hybrid_property(desc)
 
-        eq_(set(inspect(AnonClass).attrs.keys()), set(["id", "name", "q"]))
+        eq_(set(inspect(AnonClass).attrs.keys()), {"id", "name", "q"})
         eq_(
             set(inspect(AnonClass).all_orm_descriptors.keys()),
-            set(["id", "name", "q", "foob"]),
+            {"id", "name", "q", "foob"},
         )
 
     def _random_names(self):
@@ -453,9 +456,9 @@ class TestORMInspection(_fixtures.FixtureTest):
         names = self._random_names()
 
         if base is supercls:
-            pk_names = set(
+            pk_names = {
                 random.choice(names) for i in range(random.randint(1, 3))
-            )
+            }
             fk_name = random.choice(
                 [name for name in names if name not in pk_names]
             )
@@ -511,9 +514,8 @@ class %s(SuperCls):
         exec(code, glbls)
         return names, glbls[clsname]
 
-    @testing.requires.pep520
     def test_all_orm_descriptors_pep520_noinh(self):
-        from sqlalchemy.ext.declarative import declarative_base
+        from sqlalchemy.orm import declarative_base
 
         Base = declarative_base()
 
@@ -524,9 +526,8 @@ class %s(SuperCls):
 
         eq_(MyClass.__mapper__.all_orm_descriptors.keys(), names)
 
-    @testing.requires.pep520
     def test_all_orm_descriptors_pep520_onelevel_inh(self):
-        from sqlalchemy.ext.declarative import declarative_base
+        from sqlalchemy.orm import declarative_base
 
         Base = declarative_base()
 
@@ -545,12 +546,10 @@ class %s(SuperCls):
             sub_names + base_names,
         )
 
-    @testing.requires.pep520
     def test_all_orm_descriptors_pep520_classical(self):
-        class MyClass(object):
+        class MyClass:
             pass
 
-        from sqlalchemy.orm import mapper
         from sqlalchemy import Table, MetaData, Column, Integer
 
         names = self._random_names()
@@ -560,10 +559,10 @@ class %s(SuperCls):
             "t",
             m,
             Column("id", Integer, primary_key=True),
-            *[Column(name, Integer) for name in names]
+            *[Column(name, Integer) for name in names],
         )
 
-        m = mapper(MyClass, t)
+        m = self.mapper_registry.map_imperatively(MyClass, t)
 
         eq_(m.all_orm_descriptors.keys(), ["id"] + names)
 
@@ -581,7 +580,7 @@ class %s(SuperCls):
         s.flush()
         insp = inspect(u1)
         eq_(insp.identity, (u1.id,))
-        is_(s.query(User).get(insp.identity), u1)
+        is_(s.get(User, insp.identity), u1)
 
     def test_is_instance(self):
         User = self.classes.User
@@ -638,7 +637,7 @@ class %s(SuperCls):
         insp = inspect(u1)
 
         is_(insp.session, None)
-        s = Session()
+        s = fixture_session()
         s.add(u1)
         is_(insp.session, s)
 
