@@ -1010,7 +1010,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         self._assert_eager_with_entity_exception(
             [User],
             lambda: (joinedload(User.addresses).joinedload(User.orders),),
-            r'ORM mapped attribute "User.orders" does not link '
+            r'ORM mapped entity or attribute "User.orders" does not link '
             r'from relationship "User.addresses"',
         )
 
@@ -1024,7 +1024,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
                     User.orders.of_type(Order)
                 ),
             ),
-            r'ORM mapped attribute "User.orders" does not link '
+            r'ORM mapped entity or attribute "User.orders" does not link '
             r'from relationship "User.addresses"',
         )
 
@@ -1151,7 +1151,8 @@ class OptionsNoPropTestInh(_Polymorphic):
         e1 = with_polymorphic(Person, [Engineer])
         assert_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "Manager.manager_name" does not link from '
+            r'ORM mapped entity or attribute "Manager.manager_name" does '
+            r"not link from "
             r'relationship "Company.employees.'
             r'of_type\(with_polymorphic\(Person, \[Engineer\]\)\)".$',
             lambda: s.query(Company)
@@ -1168,7 +1169,8 @@ class OptionsNoPropTestInh(_Polymorphic):
 
         assert_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "Manager.manager_name" does not link from '
+            r'ORM mapped entity or attribute "Manager.manager_name" does '
+            r"not link from "
             r'relationship "Company.employees.'
             r'of_type\(Mapper\[Engineer\(engineers\)\]\)".$',
             lambda: s.query(Company)
@@ -1187,7 +1189,8 @@ class OptionsNoPropTestInh(_Polymorphic):
         # that doesn't get mixed up here
         assert_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "Manager.status" does not link from '
+            r'ORM mapped entity or attribute "Manager.status" does '
+            r"not link from "
             r'relationship "Company.employees.'
             r'of_type\(Mapper\[Engineer\(engineers\)\]\)".$',
             lambda: s.query(Company)
@@ -1206,7 +1209,8 @@ class OptionsNoPropTestInh(_Polymorphic):
 
         assert_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "Manager.manager_name" does not link from '
+            r'ORM mapped entity or attribute "Manager.manager_name" does '
+            r"not link from "
             r'relationship "Company.employees.'
             r'of_type\(with_polymorphic\(Person, \[Manager\]\)\)".$',
             lambda: s.query(Company)
@@ -1218,30 +1222,28 @@ class OptionsNoPropTestInh(_Polymorphic):
             ._compile_state(),
         )
 
-    def test_missing_attr_is_missing_of_type_for_alias(self):
+    @testing.variation("use_options", [True, False])
+    def test_missing_attr_is_missing_of_type_for_subtype(self, use_options):
         s = fixture_session()
 
-        pa = aliased(Person)
-
-        assert_raises_message(
+        with expect_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "aliased\(Person\).name" does not link '
-            r'from relationship "Company.employees".  Did you mean to use '
-            r'"Company.employees.of_type\(aliased\(Person\)\)\"?',
-            lambda: s.query(Company)
-            .options(joinedload(Company.employees).load_only(pa.name))
-            ._compile_state(),
-        )
+            r"ORM mapped entity or attribute "
+            r'(?:"Mapper\[Engineer\(engineers\)\]"|"Engineer.engineer_name") '
+            r'does not link from relationship "Company.employees".  Did you '
+            r'mean to use "Company.employees.of_type\(Engineer\)"\?',
+        ):
 
-        q = s.query(Company).options(
-            joinedload(Company.employees.of_type(pa)).load_only(pa.name)
-        )
-        orig_path = inspect(Company)._path_registry[
-            Company.employees.property
-        ][inspect(pa)][pa.name.property]
-        key = ("loader", orig_path.natural_path)
-        loader = q._compile_state().attributes[key]
-        eq_(loader.path, orig_path)
+            if use_options:
+                s.query(Company).options(
+                    joinedload(Company.employees).options(
+                        defer(Engineer.engineer_name)
+                    )
+                )._compile_state()
+            else:
+                s.query(Company).options(
+                    joinedload(Company.employees).defer(Engineer.engineer_name)
+                )._compile_state()
 
 
 class PickleTest(fixtures.MappedTest):
@@ -1499,7 +1501,8 @@ class SubOptionsTest(PathTest, QueryTest):
 
         with expect_raises_message(
             sa.exc.ArgumentError,
-            r'ORM mapped attribute "Item.keywords" does not link from '
+            r'ORM mapped entity or attribute "Item.keywords" does '
+            r"not link from "
             r'relationship "User.orders"',
         ):
             [
@@ -1508,8 +1511,9 @@ class SubOptionsTest(PathTest, QueryTest):
             ]
         with expect_raises_message(
             sa.exc.ArgumentError,
-            r'Attribute "Item.keywords" does not link from '
-            r'element "Mapper\[Order\(orders\)\]"',
+            r'ORM mapped entity or attribute "Item.keywords" does '
+            r"not link from "
+            r'relationship "User.orders"',
         ):
             joinedload(User.orders).options(
                 joinedload(Item.keywords), joinedload(Order.items)
