@@ -59,7 +59,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         )
         self.assert_sql_execution(
             testing.db,
-            lambda: metadata.create_all(checkfirst=False),
+            lambda: metadata.create_all(testing.db, checkfirst=False),
             CompiledSQL(
                 "CREATE TABLE employees ("
                 "id INTEGER NOT NULL, "
@@ -292,7 +292,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         assertions.append(AllOf(*fk_assertions))
 
         with self.sql_execution_asserter() as asserter:
-            metadata.create_all(checkfirst=False)
+            metadata.create_all(testing.db, checkfirst=False)
         asserter.assert_(*assertions)
 
         assertions = [
@@ -302,7 +302,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         ]
 
         with self.sql_execution_asserter() as asserter:
-            metadata.drop_all(checkfirst=False),
+            metadata.drop_all(testing.db, checkfirst=False),
         asserter.assert_(*assertions)
 
     def _assert_cyclic_constraint_no_alter(
@@ -356,7 +356,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         assertions = [AllOf(*table_assertions)]
 
         with self.sql_execution_asserter() as asserter:
-            metadata.create_all(checkfirst=False)
+            metadata.create_all(testing.db, checkfirst=False)
         asserter.assert_(*assertions)
 
         assertions = [
@@ -366,15 +366,15 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         if sqlite_warning:
             with expect_warnings("Can't sort tables for DROP; "):
                 with self.sql_execution_asserter() as asserter:
-                    metadata.drop_all(checkfirst=False),
+                    metadata.drop_all(testing.db, checkfirst=False),
         else:
             with self.sql_execution_asserter() as asserter:
-                metadata.drop_all(checkfirst=False),
+                metadata.drop_all(testing.db, checkfirst=False),
         asserter.assert_(*assertions)
 
     @testing.force_drop_names("a", "b")
     def test_cycle_unnamed_fks(self):
-        metadata = MetaData(testing.db)
+        metadata = MetaData()
 
         Table(
             "a",
@@ -417,7 +417,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
             ),
         ]
         with self.sql_execution_asserter() as asserter:
-            metadata.create_all(checkfirst=False)
+            metadata.create_all(testing.db, checkfirst=False)
 
         if testing.db.dialect.supports_alter:
             asserter.assert_(*assertions)
@@ -431,6 +431,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
                 "cycle have names so that they can be dropped using "
                 "DROP CONSTRAINT.",
                 metadata.drop_all,
+                testing.db,
                 checkfirst=False,
             )
         else:
@@ -439,7 +440,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
                 "foreign key dependency exists between tables"
             ):
                 with self.sql_execution_asserter() as asserter:
-                    metadata.drop_all(checkfirst=False)
+                    metadata.drop_all(testing.db, checkfirst=False)
 
             asserter.assert_(
                 AllOf(CompiledSQL("DROP TABLE b"), CompiledSQL("DROP TABLE a"))
@@ -447,7 +448,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
 
     @testing.force_drop_names("a", "b")
     def test_cycle_named_fks(self):
-        metadata = MetaData(testing.db)
+        metadata = MetaData()
 
         Table(
             "a",
@@ -491,13 +492,13 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
             ),
         ]
         with self.sql_execution_asserter() as asserter:
-            metadata.create_all(checkfirst=False)
+            metadata.create_all(testing.db, checkfirst=False)
 
         if testing.db.dialect.supports_alter:
             asserter.assert_(*assertions)
 
             with self.sql_execution_asserter() as asserter:
-                metadata.drop_all(checkfirst=False)
+                metadata.drop_all(testing.db, checkfirst=False)
 
             asserter.assert_(
                 CompiledSQL("ALTER TABLE b DROP CONSTRAINT aidfk"),
@@ -507,7 +508,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
             )
         else:
             with self.sql_execution_asserter() as asserter:
-                metadata.drop_all(checkfirst=False)
+                metadata.drop_all(testing.db, checkfirst=False)
 
             asserter.assert_(
                 AllOf(CompiledSQL("DROP TABLE b"), CompiledSQL("DROP TABLE a"))
@@ -536,7 +537,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
 
         self.assert_sql_execution(
             testing.db,
-            lambda: metadata.create_all(checkfirst=False),
+            lambda: metadata.create_all(testing.db, checkfirst=False),
             AllOf(
                 CompiledSQL(
                     "CREATE TABLE foo ("
@@ -579,7 +580,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
 
         self.assert_sql_execution(
             testing.db,
-            lambda: metadata.create_all(checkfirst=False),
+            lambda: metadata.create_all(testing.db, checkfirst=False),
             AllOf(
                 CompiledSQL(
                     "CREATE TABLE foo ("
@@ -628,7 +629,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
 
         self.assert_sql_execution(
             testing.db,
-            lambda: metadata.create_all(checkfirst=False),
+            lambda: metadata.create_all(testing.db, checkfirst=False),
             RegexSQL("^CREATE TABLE"),
             AllOf(
                 CompiledSQL(
@@ -665,7 +666,7 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
 
         self.assert_sql_execution(
             testing.db,
-            lambda: metadata.create_all(checkfirst=False),
+            lambda: metadata.create_all(testing.db, checkfirst=False),
             RegexSQL("^CREATE TABLE"),
             AllOf(
                 CompiledSQL(
@@ -704,15 +705,13 @@ class ConstraintGenTest(fixtures.TestBase, AssertsExecutionResults):
         Index("idx_winners", events.c.winner)
 
         eq_(
-            set(ix.name for ix in events.indexes),
-            set(
-                [
-                    "ix_events_name",
-                    "ix_events_location",
-                    "sport_announcer",
-                    "idx_winners",
-                ]
-            ),
+            {ix.name for ix in events.indexes},
+            {
+                "ix_events_name",
+                "ix_events_location",
+                "sport_announcer",
+                "idx_winners",
+            },
         )
 
         self.assert_sql_execution(
@@ -764,6 +763,14 @@ class ConstraintCompilationTest(fixtures.TestBase, AssertsCompiledSQL):
         i = Index("xyz", t.c.x)
         self.assert_compile(schema.CreateIndex(i), "CREATE INDEX xyz ON t (x)")
 
+    def test_create_index_if_not_exists(self):
+        t = Table("t", MetaData(), Column("x", Integer))
+        i = Index("xyz", t.c.x)
+        self.assert_compile(
+            schema.CreateIndex(i, if_not_exists=True),
+            "CREATE INDEX IF NOT EXISTS xyz ON t (x)",
+        )
+
     def test_drop_index_plain_unattached(self):
         self.assert_compile(
             schema.DropIndex(Index(name="xyz")), "DROP INDEX xyz"
@@ -774,6 +781,12 @@ class ConstraintCompilationTest(fixtures.TestBase, AssertsCompiledSQL):
             schema.DropIndex(Index(name="xyz")), "DROP INDEX xyz"
         )
 
+    def test_drop_index_if_exists(self):
+        self.assert_compile(
+            schema.DropIndex(Index(name="xyz"), if_exists=True),
+            "DROP INDEX IF EXISTS xyz",
+        )
+
     def test_create_index_schema(self):
         t = Table("t", MetaData(), Column("x", Integer), schema="foo")
         i = Index("xyz", t.c.x)
@@ -823,20 +836,46 @@ class ConstraintCompilationTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect=dialect,
         )
 
-    def test_functional_index(self):
+    def test_functional_index_w_string_cols_combo(self):
         metadata = MetaData()
-        x = Table("x", metadata, Column("q", String(50)))
-        idx = Index("y", func.lower(x.c.q))
-
-        self.assert_compile(
-            schema.CreateIndex(idx), "CREATE INDEX y ON x (lower(q))"
+        x = Table(
+            "x",
+            metadata,
+            Column("q", String(50)),
+            Column("p", Integer),
+            Column("z", Integer),
         )
 
-        self.assert_compile(
-            schema.CreateIndex(idx),
-            "CREATE INDEX y ON x (lower(q))",
-            dialect=testing.db.dialect,
-        )
+        for idx, ddl in [
+            (
+                Index("y", func.lower(x.c.q), "p", x.c.z),
+                "CREATE INDEX y ON x (lower(q), p, z)",
+            ),
+            (
+                Index("y", "p", func.lower(x.c.q), "z"),
+                "CREATE INDEX y ON x (p, lower(q), z)",
+            ),
+            (
+                Index("y", "p", "z", func.lower(x.c.q)),
+                "CREATE INDEX y ON x (p, z, lower(q))",
+            ),
+            (
+                Index("y", func.foo("foob"), x.c.p, "z"),
+                "CREATE INDEX y ON x (foo('foob'), p, z)",
+            ),
+            (
+                Index("y", x.c.p, func.foo("foob"), "z"),
+                "CREATE INDEX y ON x (p, foo('foob'), z)",
+            ),
+            (
+                Index("y", func.foo("foob"), "p", "z"),
+                "CREATE INDEX y ON x (foo('foob'), p, z)",
+            ),
+        ]:
+            x.append_constraint(idx)
+            self.assert_compile(schema.CreateIndex(idx), ddl)
+
+            x.to_metadata(MetaData())
 
     def test_index_against_text_separate(self):
         metadata = MetaData()
@@ -1001,7 +1040,7 @@ class ConstraintCompilationTest(fixtures.TestBase, AssertsCompiledSQL):
             Column("a", Integer),
             Column("b", Integer),
             Column("c", Integer),
-            *fkcs
+            *fkcs,
         )
         Table("remote", m, Column("id", Integer, primary_key=True))
 
@@ -1234,6 +1273,16 @@ class ConstraintCompilationTest(fixtures.TestBase, AssertsCompiledSQL):
             "ALTER TABLE tbl DROP CONSTRAINT my_test_constraint CASCADE",
         )
 
+    def test_render_drop_constraint_if_exists(self):
+        t, t2 = self._constraint_create_fixture()
+
+        constraint = CheckConstraint("a = 1", name="a1", table=t)
+
+        self.assert_compile(
+            schema.DropConstraint(constraint, if_exists=True),
+            "ALTER TABLE tbl DROP CONSTRAINT IF EXISTS a1",
+        )
+
     def test_render_add_fk_constraint_stringcol(self):
         t, t2 = self._constraint_create_fixture()
 
