@@ -13,24 +13,23 @@ from sqlalchemy import MetaData
 from sqlalchemy import Sequence
 from sqlalchemy import String
 from sqlalchemy import testing
-from sqlalchemy import Unicode
 from sqlalchemy.schema import CreateTable
 from sqlalchemy.sql import literal_column
 from sqlalchemy.sql import select
 from sqlalchemy.sql import text
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
-from sqlalchemy.testing import engines
+from sqlalchemy.testing import config
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import mock
+from sqlalchemy.testing.assertions import expect_deprecated
+from sqlalchemy.testing.provision import normalize_sequence
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.types import TypeDecorator
 from sqlalchemy.types import TypeEngine
-from sqlalchemy.util import b
-from sqlalchemy.util import u
 
 
 class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
@@ -144,11 +143,11 @@ class DefaultObjectTest(fixtures.TestBase):
         def fn2(x, y, z=3):
             pass
 
-        class fn3(object):
+        class fn3:
             def __init__(self, x, y):
                 pass
 
-        class FN4(object):
+        class FN4:
             def __call__(self, x, y):
                 pass
 
@@ -174,21 +173,21 @@ class DefaultObjectTest(fixtures.TestBase):
 
         fn5 = list
 
-        class fn6a(object):
+        class fn6a:
             def __init__(self, x):
                 eq_(x, "context")
 
-        class fn6b(object):
+        class fn6b:
             def __init__(self, x, y=3):
                 eq_(x, "context")
 
-        class FN7(object):
+        class FN7:
             def __call__(self, x):
                 eq_(x, "context")
 
         fn7 = FN7()
 
-        class FN8(object):
+        class FN8:
             def __call__(self, x, y=3):
                 eq_(x, "context")
 
@@ -358,7 +357,8 @@ class DefaultObjectTest(fixtures.TestBase):
             assert_raises_message(
                 sa.exc.ArgumentError,
                 r"SQL expression for WHERE/HAVING role expected, "
-                r"got (?:Sequence|ColumnDefault|DefaultClause)\('y'.*\)",
+                r"got (?:Sequence|(?:ScalarElement)ColumnDefault|"
+                r"DefaultClause)\('y'.*\)",
                 t.select().where,
                 const,
             )
@@ -400,13 +400,14 @@ class DefaultRoundTripTest(fixtures.TablesTest):
         use_function_defaults = testing.against("postgresql", "mssql")
         is_oracle = testing.against("oracle")
 
-        class MyClass(object):
+        class MyClass:
             @classmethod
             def gen_default(cls, ctx):
                 return "hi"
 
         class MyType(TypeDecorator):
             impl = String(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -505,15 +506,45 @@ class DefaultRoundTripTest(fixtures.TablesTest):
     def teardown_test(self):
         self.default_generator["x"] = 50
 
-    def test_standalone(self, connection):
+    def test_standalone_via_exec_removed(self, connection):
         t = self.tables.default_test
-        x = connection.execute(t.c.col1.default)
-        y = connection.execute(t.c.col2.default)
-        z = connection.execute(t.c.col3.default)
+
+        with expect_deprecated(
+            r"Using the .execute\(\) method to invoke a "
+            r"DefaultGenerator object is deprecated; please use "
+            r"the .scalar\(\) method."
+        ):
+            connection.execute(t.c.col1.default)
+        with expect_deprecated(
+            r"Using the .execute\(\) method to invoke a "
+            r"DefaultGenerator object is deprecated; please use "
+            r"the .scalar\(\) method."
+        ):
+            connection.execute(t.c.col2.default)
+        with expect_deprecated(
+            r"Using the .execute\(\) method to invoke a "
+            r"DefaultGenerator object is deprecated; please use "
+            r"the .scalar\(\) method."
+        ):
+            connection.execute(t.c.col3.default)
+
+    def test_standalone_default_scalar(self, connection):
+        t = self.tables.default_test
+        x = connection.scalar(t.c.col1.default)
+        y = connection.scalar(t.c.col2.default)
+        z = connection.scalar(t.c.col3.default)
         assert 50 <= x <= 57
         eq_(y, "imthedefault")
         eq_(z, self.f)
 
+    def test_standalone_function_execute(self, connection):
+        ctexec = connection.execute(self.currenttime)
+        assert isinstance(ctexec.scalar(), datetime.date)
+
+    def test_standalone_function_scalar(self, connection):
+        ctexec = connection.scalar(self.currenttime)
+        assert isinstance(ctexec, datetime.date)
+
     def test_insert(self, connection):
         t = self.tables.default_test
 
@@ -521,14 +552,14 @@ class DefaultRoundTripTest(fixtures.TablesTest):
         assert r.lastrow_has_defaults()
         eq_(
             set(r.context.postfetch_cols),
-            set([t.c.col3, t.c.col5, t.c.col4, t.c.col6]),
+            {t.c.col3, t.c.col5, t.c.col4, t.c.col6},
         )
 
         r = connection.execute(t.insert().inline())
         assert r.lastrow_has_defaults()
         eq_(
             set(r.context.postfetch_cols),
-            set([t.c.col3, t.c.col5, t.c.col4, t.c.col6]),
+            {t.c.col3, t.c.col5, t.c.col4, t.c.col6},
         )
 
         connection.execute(t.insert())
@@ -568,7 +599,7 @@ class DefaultRoundTripTest(fixtures.TablesTest):
 
         eq_(
             set(r.context.postfetch_cols),
-            set([t.c.col3, t.c.col5, t.c.col4, t.c.col6]),
+            {t.c.col3, t.c.col5, t.c.col4, t.c.col6},
         )
 
         eq_(
@@ -840,15 +871,8 @@ class DefaultRoundTripTest(fixtures.TablesTest):
         eq_(55, row._mapping["col3"])
 
 
-class FutureDefaultRoundTripTest(
-    fixtures.FutureEngineMixin, DefaultRoundTripTest
-):
-
-    __backend__ = True
-
-
 class CTEDefaultTest(fixtures.TablesTest):
-    __requires__ = ("ctes", "returning", "ctes_on_dml")
+    __requires__ = ("ctes", "insert_returning", "ctes_on_dml")
     __backend__ = True
 
     @classmethod
@@ -927,68 +951,73 @@ class CTEDefaultTest(fixtures.TablesTest):
         eq_(conn.execute(select(q.c.x, q.c.y)).first(), expected)
 
 
-class PKDefaultTest(fixtures.TablesTest):
+class PKDefaultTest(fixtures.TestBase):
     __requires__ = ("subqueries",)
     __backend__ = True
 
-    @classmethod
-    def define_tables(cls, metadata):
-        t2 = Table("t2", metadata, Column("nextid", Integer))
-
-        Table(
-            "t1",
-            metadata,
-            Column(
-                "id",
-                Integer,
-                primary_key=True,
-                default=sa.select(func.max(t2.c.nextid)).scalar_subquery(),
-            ),
-            Column("data", String(30)),
-        )
-
-        Table(
-            "date_table",
-            metadata,
-            Column(
-                "date_id",
-                DateTime(timezone=True),
-                default=text("current_timestamp"),
-                primary_key=True,
-            ),
-        )
-
-    @testing.requires.returning
-    def test_with_implicit_returning(self):
-        self._test(True)
-
-    def test_regular(self):
-        self._test(False)
-
-    def _test(self, returning):
-        t2, t1, date_table = (
-            self.tables.t2,
-            self.tables.t1,
-            self.tables.date_table,
-        )
-
-        if not returning and not testing.db.dialect.implicit_returning:
-            engine = testing.db
-        else:
-            engine = engines.testing_engine(
-                options={"implicit_returning": returning}
+    @testing.fixture
+    def table_fixture(self, metadata, connection):
+        def go(implicit_returning):
+            t2 = Table(
+                "t2",
+                metadata,
+                Column("nextid", Integer),
+                implicit_returning=implicit_returning,
             )
-        with engine.begin() as conn:
-            conn.execute(t2.insert(), dict(nextid=1))
-            r = conn.execute(t1.insert(), dict(data="hi"))
-            eq_((1,), r.inserted_primary_key)
 
-            conn.execute(t2.insert(), dict(nextid=2))
-            r = conn.execute(t1.insert(), dict(data="there"))
-            eq_((2,), r.inserted_primary_key)
+            t1 = Table(
+                "t1",
+                metadata,
+                Column(
+                    "id",
+                    Integer,
+                    primary_key=True,
+                    default=sa.select(func.max(t2.c.nextid)).scalar_subquery(),
+                ),
+                Column("data", String(30)),
+                implicit_returning=implicit_returning,
+            )
 
-            r = conn.execute(date_table.insert())
-            assert isinstance(r.inserted_primary_key[0], datetime.datetime)
+            date_table = Table(
+                "date_table",
+                metadata,
+                Column(
+                    "date_id",
+                    DateTime(timezone=True),
+                    default=text("current_timestamp"),
+                    primary_key=True,
+                ),
+                implicit_returning=implicit_returning,
+            )
+
+            metadata.create_all(connection)
+            return t1, t2, date_table
+
+        return go
+
+    @testing.crashes(
+        "+mariadbconnector", "https://jira.mariadb.org/browse/CONPY-206"
+    )
+    @testing.combinations(
+        (True, testing.requires.insert_returning),
+        (False,),
+        argnames="implicit_returning",
+    )
+    def test_pk_default(self, connection, table_fixture, implicit_returning):
+        t1, t2, date_table = table_fixture(implicit_returning)
+
+        conn = connection
+
+        conn.execute(t2.insert(), dict(nextid=1))
+        r = conn.execute(t1.insert(), dict(data="hi"))
+        eq_((1,), r.inserted_primary_key)
+
+        conn.execute(t2.insert(), dict(nextid=2))
+        r = conn.execute(t1.insert(), dict(data="there"))
+        eq_((2,), r.inserted_primary_key)
+
+        r = conn.execute(date_table.insert())
+        assert isinstance(r.inserted_primary_key[0], datetime.datetime)
 
 
 class PKIncrementTest(fixtures.TablesTest):
@@ -1003,7 +1032,9 @@ class PKIncrementTest(fixtures.TablesTest):
             Column(
                 "id",
                 Integer,
-                Sequence("ai_id_seq", optional=True),
+                normalize_sequence(
+                    config, Sequence("ai_id_seq", optional=True)
+                ),
                 primary_key=True,
             ),
             Column("int1", Integer),
@@ -1084,6 +1115,7 @@ class AutoIncrementTest(fixtures.TestBase):
     def test_autoinc_detection_no_affinity(self):
         class MyType(TypeDecorator):
             impl = TypeEngine
+            cache_ok = True
 
         assert MyType()._type_affinity is None
         t = Table("x", MetaData(), Column("id", MyType(), primary_key=True))
@@ -1143,7 +1175,7 @@ class AutoIncrementTest(fixtures.TestBase):
         # and autoincrement=False.  Using a ForeignKey
         # would have the same effect
 
-        some_seq = Sequence("some_seq")
+        some_seq = normalize_sequence(config, Sequence("some_seq"))
 
         dataset_no_autoinc = Table(
             "x",
@@ -1212,6 +1244,8 @@ class SpecialTypePKTest(fixtures.TestBase):
         class MyInteger(TypeDecorator):
             impl = Integer
 
+            cache_ok = True
+
             def process_bind_param(self, value, dialect):
                 if value is None:
                     return None
@@ -1251,7 +1285,7 @@ class SpecialTypePKTest(fixtures.TestBase):
 
             # we don't pre-fetch 'server_default'.
             if "server_default" in kw and (
-                not testing.db.dialect.implicit_returning
+                not testing.db.dialect.insert_returning
                 or not implicit_returning
             ):
                 eq_(r.inserted_primary_key, (None,))
@@ -1283,7 +1317,7 @@ class SpecialTypePKTest(fixtures.TestBase):
         self._run_test(default=literal_column("1", type_=self.MyInteger))
 
     def test_sequence(self):
-        self._run_test(Sequence("foo_seq"))
+        self._run_test(normalize_sequence(config, Sequence("foo_seq")))
 
     def test_text_clause_default_no_type(self):
         self._run_test(default=text("1"))
@@ -1294,15 +1328,18 @@ class SpecialTypePKTest(fixtures.TestBase):
     def test_server_default_no_autoincrement(self):
         self._run_test(server_default="1", autoincrement=False)
 
+    @testing.crashes(
+        "+mariadbconnector", "https://jira.mariadb.org/browse/CONPY-206"
+    )
     def test_clause(self):
         stmt = select(cast("INT_1", type_=self.MyInteger)).scalar_subquery()
         self._run_test(default=stmt)
 
-    @testing.requires.returning
+    @testing.requires.insert_returning
     def test_no_implicit_returning(self):
         self._run_test(implicit_returning=False)
 
-    @testing.requires.returning
+    @testing.requires.insert_returning
     def test_server_default_no_implicit_returning(self):
         self._run_test(server_default="1", autoincrement=False)
 
@@ -1336,7 +1373,7 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         eq_(r.inserted_primary_key, (None,))
         eq_(list(connection.execute(t.select())), [("key_one", "data")])
 
-    @testing.requires.returning
+    @testing.requires.insert_returning
     @testing.provide_metadata
     def test_string_default_on_insert_with_returning(self, connection):
         """With implicit_returning, we get a string PK default back no
@@ -1414,8 +1451,9 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         else:
             eq_(list(connection.execute(t2.select())), [(5, "data")])
 
-    @testing.requires.returning
+    @testing.requires.insert_returning
     @testing.provide_metadata
+    @testing.fails_on("sqlite", "sqlite doesn't like our default trick here")
     def test_int_default_on_insert_with_returning(self, connection):
         metadata = self.metadata
         t = Table(
@@ -1431,29 +1469,6 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         eq_(list(connection.execute(t.select())), [(5, "data")])
 
 
-class UnicodeDefaultsTest(fixtures.TestBase):
-    __backend__ = True
-
-    def test_no_default(self):
-        Column(Unicode(32))
-
-    def test_unicode_default(self):
-        default = u("foo")
-        Column(Unicode(32), default=default)
-
-    def test_nonunicode_default(self):
-        default = b("foo")
-        assert_raises_message(
-            sa.exc.SAWarning,
-            "Unicode column 'foobar' has non-unicode "
-            "default value b?'foo' specified.",
-            Column,
-            "foobar",
-            Unicode(32),
-            default=default,
-        )
-
-
 class InsertFromSelectTest(fixtures.TablesTest):
     __backend__ = True
 
