@@ -6,6 +6,7 @@ from sqlalchemy import bindparam
 from sqlalchemy import Column
 from sqlalchemy import desc
 from sqlalchemy import exc
+from sqlalchemy import extract
 from sqlalchemy import Float
 from sqlalchemy import func
 from sqlalchemy import Integer
@@ -31,6 +32,7 @@ from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.assertions import expect_raises_message
 from sqlalchemy.types import NullType
 
 table1 = table(
@@ -177,8 +179,17 @@ class SelectCompositionTest(fixtures.TestBase, AssertsCompiledSQL):
             select(table1.alias("t"), text("foo.f"))
             .where(text("foo.f = t.id"))
             .select_from(text("(select f from bar where lala=heyhey) foo")),
-            "SELECT t.myid, t.name, t.description, foo.f FROM mytable AS t, "
-            "(select f from bar where lala=heyhey) foo WHERE foo.f = t.id",
+            "SELECT t.myid, t.name, t.description, foo.f FROM "
+            "(select f from bar where lala=heyhey) foo, "
+            "mytable AS t WHERE foo.f = t.id",
+        )
+
+    def test_expression_element_role(self):
+        """test #7287"""
+
+        self.assert_compile(
+            extract("year", text("some_date + :param")),
+            "EXTRACT(year FROM some_date + :param)",
         )
 
     @testing.combinations(
@@ -363,7 +374,7 @@ class BindParamTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
     def _assert_type_map(self, t, compare):
-        map_ = dict((b.key, b.type) for b in t._bindparams.values())
+        map_ = {b.key: b.type for b in t._bindparams.values()}
         for k in compare:
             assert compare[k]._type_affinity is map_[k]._type_affinity
 
@@ -631,11 +642,11 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def _mapping(self, stmt):
         compiled = stmt.compile()
-        return dict(
-            (elem, key)
+        return {
+            elem: key
             for key, elements in compiled._create_result_map().items()
             for elem in elements[1]
-        )
+        }
 
     def test_select_label_alt_name(self):
         t = self._xy_table_fixture()
@@ -678,6 +689,19 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
         mapping = self._mapping(s)
         assert x not in mapping
 
+    def test_subquery_accessors(self):
+        t = self._xy_table_fixture()
+
+        s = text("SELECT x from t").columns(t.c.x)
+
+        self.assert_compile(
+            select(s.scalar_subquery()), "SELECT (SELECT x from t) AS anon_1"
+        )
+        self.assert_compile(
+            select(s.subquery()),
+            "SELECT anon_1.x FROM (SELECT x from t) AS anon_1",
+        )
+
     def test_select_label_alt_name_table_alias_column(self):
         t = self._xy_table_fixture()
         x = t.c.x
@@ -706,6 +730,36 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
             "FROM mytable, t WHERE mytable.myid = t.id",
         )
 
+    def test_cte_recursive(self):
+        t = (
+            text("select id, name from user")
+            .columns(id=Integer, name=String)
+            .cte("t", recursive=True)
+        )
+
+        s = select(table1).where(table1.c.myid == t.c.id)
+        self.assert_compile(
+            s,
+            "WITH RECURSIVE t(id, name) AS (select id, name from user) "
+            "SELECT mytable.myid, mytable.name, mytable.description "
+            "FROM mytable, t WHERE mytable.myid = t.id",
+        )
+
+    def test_unions(self):
+        s1 = text("select id, name from user where id > 5").columns(
+            id=Integer, name=String
+        )
+        s2 = text("select id, name from user where id < 15").columns(
+            id=Integer, name=String
+        )
+        stmt = union(s1, s2)
+        eq_(stmt.selected_columns.keys(), ["id", "name"])
+        self.assert_compile(
+            stmt,
+            "select id, name from user where id > 5 UNION "
+            "select id, name from user where id < 15",
+        )
+
     def test_subquery(self):
         t = (
             text("select id, name from user")
@@ -761,20 +815,22 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
         t = t.bindparams(bar=String)
         t = t.bindparams(bindparam("bat", value="bat"))
 
-        eq_(set(t.element._bindparams), set(["bat", "foo", "bar"]))
+        eq_(set(t.element._bindparams), {"bat", "foo", "bar"})
 
 
 class TextErrorsTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
     def _test(self, fn, arg, offending_clause):
+        arg = util.to_list(arg)
+
         assert_raises_message(
             exc.ArgumentError,
             r"Textual (?:SQL|column|SQL FROM) expression %(stmt)r should be "
             r"explicitly declared (?:with|as) text\(%(stmt)r\)"
             % {"stmt": util.ellipses_string(offending_clause)},
             fn,
-            arg,
+            *arg,
         )
 
     def test_where(self):
@@ -811,6 +867,15 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
             stmt, "SELECT mytable.myid AS foo FROM mytable ORDER BY foo"
         )
 
+    def test_no_order_by_text(self):
+        stmt = select(text("foo")).order_by("foo")
+
+        with expect_raises_message(
+            exc.CompileError,
+            r"Can't resolve label reference for ORDER BY / GROUP BY / ",
+        ):
+            stmt.compile()
+
     def test_order_by_colname(self):
         stmt = select(table1.c.myid).order_by("name")
         self.assert_compile(
@@ -959,7 +1024,7 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
             '"SUM(ABC)_"',
         )
 
-    def test_order_by_literal_col_quoting_one_explict_quote(self):
+    def test_order_by_literal_col_quoting_one_explicit_quote(self):
         col = literal_column("SUM(ABC)").label(quoted_name("SUM(ABC)", True))
         tbl = table("my_table")
         query = select(col).select_from(tbl).order_by(col)
